"""Auto-generated file, do not edit by hand. IT metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_IT = PhoneMetadata(id='IT', country_code=39, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[0389]\\d{5,10}', possible_length=(6, 9, 10, 11)),
    fixed_line=PhoneNumberDesc(national_number_pattern='0\\d{9,10}', example_number='0123456789', possible_length=(10, 11)),
    mobile=PhoneNumberDesc(national_number_pattern='3\\d{8,9}', example_number='3123456789', possible_length=(9, 10)),
    toll_free=PhoneNumberDesc(national_number_pattern='80(?:0\\d{6}|3\\d{3})', example_number='800123456', possible_length=(6, 9)),
    premium_rate=PhoneNumberDesc(national_number_pattern='89(?:2\\d{3}|9\\d{6})', example_number='892123', possible_length=(6, 9)),
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{4})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['0[26]']),
        NumberFormat(pattern='(\\d{3})(\\d{4})(\\d{3,4})', format='\\1 \\2 \\3', leading_digits_pattern=['0[13-57-9]']),
        NumberFormat(pattern='(\\d{3})(\\d{3})(\\d{3,4})', format='\\1 \\2 \\3', leading_digits_pattern=['3']),
        NumberFormat(pattern='(\\d{3})(\\d{3,6})', format='\\1 \\2', leading_digits_pattern=['8'])])



"""Auto-generated file, do not edit by hand. MQ metadata"""
from ..phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_MQ = PhoneMetadata(id='MQ', country_code=596, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='[56]\\d{8}', possible_length=(9,)),
    fixed_line=PhoneNumberDesc(national_number_pattern='596(?:0[2-5]|[12]0|3[05-9]|4[024-8]|[5-7]\\d|89|9[4-8])\\d{4}', example_number='596301234', possible_length=(9,)),
    mobile=PhoneNumberDesc(national_number_pattern='696(?:[0-47-9]\\d|5[0-6]|6[0-4])\\d{4}', example_number='696201234', possible_length=(9,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{2})(\\d{2})', format='\\1 \\2 \\3 \\4', national_prefix_formatting_rule='0\\1')])



"""Auto-generated file, do not edit by hand. JP metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_JP = PhoneMetadata(id='JP', country_code=81, international_prefix='010',
    general_desc=PhoneNumberDesc(national_number_pattern='07\\d{5}|[1-357-9]\\d{3,10}', possible_length=(4, 5, 6, 7, 8, 9, 10, 11)),
    fixed_line=PhoneNumberDesc(national_number_pattern='07\\d{5}|[1-357-9]\\d{3,10}', example_number='0712345', possible_length=(4, 5, 6, 7, 8, 9, 10, 11)),
    toll_free=PhoneNumberDesc(national_number_pattern='0777[01]\\d{2}', example_number='0777012', possible_length=(7,)),
    no_international_dialling=PhoneNumberDesc(national_number_pattern='[23]\\d{3}', example_number='2123', possible_length=(4,)),
    national_prefix='0',
    national_prefix_for_parsing='0',
    number_format=[NumberFormat(pattern='(\\d{2})(\\d{4})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['[57-9]0'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{2})(\\d{2})(\\d{3})(\\d{4})', format='\\1 \\2 \\3 \\4', leading_digits_pattern=['[57-9]0'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{2})(\\d{3})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['111|222|333', '(?:111|222|333)1', '(?:111|222|333)11'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{4})(\\d)(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['222|333', '2221|3332', '22212|3332', '222120|3332'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d{2})(\\d{4})', format='\\1 \\2 \\3', leading_digits_pattern=['[23]'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{3})(\\d{4})', format='\\1-\\2', leading_digits_pattern=['077'], national_prefix_formatting_rule='0\\1'),
        NumberFormat(pattern='(\\d{4})', format='*\\1', leading_digits_pattern=['[23]'], national_prefix_formatting_rule='\\1')])



"""Auto-generated file, do not edit by hand. TA metadata"""
from phonenumbers.phonemetadata import NumberFormat, PhoneNumberDesc, PhoneMetadata

PHONE_METADATA_TA = PhoneMetadata(id='TA', country_code=290, international_prefix='00',
    general_desc=PhoneNumberDesc(national_number_pattern='8\\d{3,7}', possible_length=(4, 6, 8)),
    fixed_line=PhoneNumberDesc(national_number_pattern='8\\d{5}', example_number='812345', possible_length=(6,)),
    mobile=PhoneNumberDesc(national_number_pattern='8\\d{3}', example_number='8123', possible_length=(4,)),
    toll_free=PhoneNumberDesc(national_number_pattern='8\\d{7}', example_number='81234567', possible_length=(8,)))



#!/usr/bin/env python
"""Unit tests for geocoder.py"""

# Based on original Java code:
#     java/test/com/google/i18n/phonenumbers/geocoding/PhoneNumberOfflineGeocoderTest.java
# Copyright (C) 2011 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from phonenumbers import PhoneNumber, FrozenPhoneNumber
from phonenumbers import geocoder
from phonenumbers.geocoder import description_for_number, country_name_for_number
from phonenumbers.geocoder import description_for_valid_number
from phonenumbers.prefix import _prefix_description_for_number
from phonenumbers.util import u

# Allow override library geocoding metadata with the test metadata.
REAL_GEOCODE_DATA = geocoder.GEOCODE_DATA
REAL_GEOCODE_LONGEST_PREFIX = geocoder.GEOCODE_LONGEST_PREFIX
from .testgeodata import GEOCODE_DATA as TEST_GEOCODE_DATA
from .testgeodata import GEOCODE_LONGEST_PREFIX as TEST_GEOCODE_LONGEST_PREFIX


def reinstate_real_geodata():
    """Reinstate real phone number geocoding metadata"""
    geocoder.GEOCODE_DATA = REAL_GEOCODE_DATA
    geocoder.GEOCODE_LONGEST_PREFIX = REAL_GEOCODE_LONGEST_PREFIX


def insert_test_geodata():
    """Insert test geocoding metadata into library"""
    geocoder.GEOCODE_DATA = TEST_GEOCODE_DATA
    geocoder.GEOCODE_LONGEST_PREFIX = TEST_GEOCODE_LONGEST_PREFIX


# Set up some test numbers to re-use.
KO_NUMBER1 = FrozenPhoneNumber(country_code=82, national_number=22123456)
KO_NUMBER2 = FrozenPhoneNumber(country_code=82, national_number=322123456)
KO_NUMBER3 = FrozenPhoneNumber(country_code=82, national_number=6421234567)
KO_INVALID_NUMBER = FrozenPhoneNumber(country_code=82, national_number=1234)
KO_MOBILE = FrozenPhoneNumber(country_code=82, national_number=101234567)
US_NUMBER1 = FrozenPhoneNumber(country_code=1, national_number=6502530000)
US_NUMBER2 = FrozenPhoneNumber(country_code=1, national_number=6509600000)
US_NUMBER3 = FrozenPhoneNumber(country_code=1, national_number=2128120000)
US_NUMBER4 = FrozenPhoneNumber(country_code=1, national_number=6174240000)
US_INVALID_NUMBER = FrozenPhoneNumber(country_code=1, national_number=123456789)
NANPA_TOLL_FREE = FrozenPhoneNumber(country_code=1, national_number=8002431234)
BS_NUMBER1 = FrozenPhoneNumber(country_code=1, national_number=2423651234)
AU_NUMBER = FrozenPhoneNumber(country_code=61, national_number=236618300)
AR_MOBILE_NUMBER = FrozenPhoneNumber(country_code=54, national_number=92214000000)
NUMBER_WITH_INVALID_COUNTRY_CODE = FrozenPhoneNumber(country_code=999, national_number=2423651234)
INTERNATIONAL_TOLL_FREE = FrozenPhoneNumber(country_code=800, national_number=12345678)

# Language/country codes
_CHINA = "CN"
_CHINESE = "zh"
_ITALIAN = "it"
_ENGLISH = "en"
_KOREAN = "ko"
_GERMAN = "de"
_FRENCH = "fr"
_SPANISH = "es"
_USA = "US"


class PhoneNumberGeocoderTest(unittest.TestCase):
    """Unit tests for geocoder.py"""

    def setUp(self):
        insert_test_geodata()

    def tearDown(self):
        reinstate_real_geodata()

    def testGetDescriptionForNumberWithNoDataFile(self):
        # No data file containing mappings for US numbers is available in Chinese for the unittests. As
        # a result, the country name of United States in simplified Chinese is returned.
        self.assertEqual(u("\u7F8E\u56FD"),
                         description_for_number(US_NUMBER1, _CHINESE, region=_CHINA))
        self.assertEqual("Bahamas",
                         description_for_number(BS_NUMBER1, _ENGLISH, region=_USA))
        self.assertEqual("Australia",
                         description_for_number(AU_NUMBER, _ENGLISH, region=_USA))
        self.assertEqual("",
                         description_for_number(NUMBER_WITH_INVALID_COUNTRY_CODE, _ENGLISH, region=_USA))
        self.assertEqual("",
                         description_for_number(INTERNATIONAL_TOLL_FREE, _ENGLISH, region=_USA))

    def testGetDescriptionForNumberWithMissingPrefix(self):
        # Test that the name of the country is returned when the number passed in
        # is valid but not covered by the geocoding data file.
        self.assertEqual("United States",
                         description_for_number(US_NUMBER4, _ENGLISH, region=_USA))

    def testGetDescriptionForNumberBelongingToMultipleCountriesIsEmpty(self):
        # Test that nothing is returned when the number passed in is valid but
        # not covered by the geocoding data file and belongs to multiple
        # countries
        self.assertEqual("", description_for_number(NANPA_TOLL_FREE, _ENGLISH, region=_USA))

    def testGetDescriptionForNumber_en_US(self):
        self.assertEqual("CA",
                         description_for_number(US_NUMBER1, _ENGLISH, region=_USA))
        self.assertEqual("Mountain View, CA",
                         description_for_number(US_NUMBER2, _ENGLISH, region=_USA))
        self.assertEqual("New York, NY",
                         description_for_number(US_NUMBER3, _ENGLISH, region=_USA))

    def testGetDescriptionForKoreanNumber(self):
        self.assertEqual("Seoul",
                         description_for_number(KO_NUMBER1, _ENGLISH))
        self.assertEqual("Incheon",
                         description_for_number(KO_NUMBER2, _ENGLISH))
        self.assertEqual("Jeju",
                         description_for_number(KO_NUMBER3, _ENGLISH))
        self.assertEqual(u("\uC11C\uC6B8"),
                         description_for_number(KO_NUMBER1, _KOREAN))
        self.assertEqual(u("\uC778\uCC9C"),
                         description_for_number(KO_NUMBER2, _KOREAN))

    def testGetDescriptionForArgentinianMobileNumber(self):
        self.assertEqual("La Plata", description_for_number(AR_MOBILE_NUMBER, _ENGLISH))
        # Python version extra test
        # Put an invalid number after the mobile token ("9") and lie about
        # this being a valid number
        arInvalidMobileNumber = PhoneNumber(country_code=54, national_number=91)
        self.assertEqual("Argentina", description_for_valid_number(arInvalidMobileNumber, _ENGLISH))

    def testGetDescriptionForFallBack(self):
        # No fallback, as the location name for the given phone number is
        # available in the requested language.
        self.assertEqual("Kalifornien",
                         description_for_number(US_NUMBER1, _GERMAN))
        # German falls back to English.
        self.assertEqual("New York, NY",
                         description_for_number(US_NUMBER3, _GERMAN))
        # Italian falls back to English.
        self.assertEqual("CA",
                         description_for_number(US_NUMBER1, _ITALIAN))
        # Korean doesn't fall back to English.
        self.assertEqual(u("\uB300\uD55C\uBBFC\uAD6D"),
                         description_for_number(KO_NUMBER3, _KOREAN))

    def testGetDescriptionForNumberWithUserRegion(self):
        # User in Italy, American number. We should just show United States, in
        # Spanish, and not more detailed information.
        self.assertEqual("Estados Unidos",
                         description_for_number(US_NUMBER1, _SPANISH, region="IT"))
        # Unknown region - should just show country name.
        self.assertEqual("Estados Unidos",
                         description_for_number(US_NUMBER1, _SPANISH, region="ZZ"))
        # User in the States, language German, should show detailed data.
        self.assertEqual("Kalifornien",
                         description_for_number(US_NUMBER1, _GERMAN, region="US"))
        # User in the States, language French, no data for French, so we fallback
        # to English detailed data.
        self.assertEqual("CA",
                         description_for_number(US_NUMBER1, _FRENCH, region="US"))
        # Invalid number - return an empty string.
        self.assertEqual("", description_for_number(US_INVALID_NUMBER, _ENGLISH, region="US"))

    def testGetDescriptionForInvalidNumber(self):
        self.assertEqual("", description_for_number(KO_INVALID_NUMBER, _ENGLISH))
        self.assertEqual("", description_for_number(US_INVALID_NUMBER, _ENGLISH))

    def testGetDescriptionForNonGeographicalNumberWithGeocodingPrefix(self):
        # We have a geocoding prefix, but we shouldn't use it since this is not geographical.
        self.assertEqual("South Korea", description_for_number(KO_MOBILE, _ENGLISH))

    def testCoverage(self):
        # Python version extra tests
        invalid_number = PhoneNumber(country_code=210, national_number=123456)
        self.assertEqual("", country_name_for_number(invalid_number, "en"))
        # Ensure we exercise all public entrypoints directly
        self.assertEqual("CA", _prefix_description_for_number(TEST_GEOCODE_DATA, TEST_GEOCODE_LONGEST_PREFIX, US_NUMBER1, "en"))
        self.assertEqual("CA", description_for_valid_number(US_NUMBER1, "en"))
        self.assertEqual("", description_for_valid_number(US_INVALID_NUMBER, "en"))
        # Add in some script and region specific fictional names
        TEST_GEOCODE_DATA['1650960'] = {'en': u("Mountain View, CA"),
                                        "en_GB": u("Mountain View California"),
                                        "en_US": u("Mountain View, Sunny California"),
                                        "en_Xyzz_US": u("MTV - xyzz"),
                                        "en_Latn": u("MountainView")}
        # The following test might one day return "Mountain View California"
        self.assertEqual("United States",
                         description_for_number(US_NUMBER2, _ENGLISH, region="GB"))
        self.assertEqual("Mountain View, Sunny California",
                         description_for_number(US_NUMBER2, _ENGLISH, region="US"))
        self.assertEqual("MountainView",
                         description_for_number(US_NUMBER2, _ENGLISH, script="Latn"))
        self.assertEqual("United States",
                         description_for_number(US_NUMBER2, _ENGLISH, script="Latn", region="GB"))
        self.assertEqual("MTV - xyzz",
                         description_for_number(US_NUMBER2, _ENGLISH, script="Xyzz", region="US"))
        self.assertEqual("Mountain View, Sunny California",
                         description_for_number(US_NUMBER2, _ENGLISH, script="Zazz", region="US"))
        # Get a different result when there is a script-specific variant
        self.assertEqual("MountainView",
                         description_for_number(US_NUMBER2, _ENGLISH, script="Latn", region="US"))
        TEST_GEOCODE_DATA['1650960'] = {'en': u("Mountain View, CA")}

        # Test the locale mapping
        TEST_GEOCODE_DATA['8868'] = {'zh': u("Chinese"), 'zh_Hant': u("Hant-specific")}
        tw_number = FrozenPhoneNumber(country_code=886, national_number=810080123)
        self.assertEqual("Hant-specific",
                         description_for_number(tw_number, "zh", region="TW"))
        del TEST_GEOCODE_DATA['8868']



#!/usr/bin/env python
"""Unit tests for phonenumbermatcher.py"""

# Based on original Java code:
#     java/test/com/google/i18n/phonenumbers/PhoneNumberMatchTest.java
#     java/test/com/google/i18n/phonenumbers/PhoneNumberMatcherTest.java
# Copyright (C) 2011 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys
import unittest

from phonenumbers import PhoneNumberMatch, PhoneNumberMatcher, Leniency
from phonenumbers import PhoneNumber, NumberFormat, phonenumberutil
from phonenumbers import phonenumbermatcher, CountryCodeSource
from phonenumbers.util import u
from .testmetadatatest import TestMetadataTestCase


class PhoneNumberMatchTest(unittest.TestCase):
    """Tests the value type semantics for PhoneNumberMatch.

    Equality must be based on the covered range and corresponding phone
    number. Range and number correctness are tested by PhoneNumberMatcherTest.
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testValueTypeSemantics(self):
        number = PhoneNumber()
        match1 = PhoneNumberMatch(10, "1 800 234 45 67", number)
        match2 = PhoneNumberMatch(10, "1 800 234 45 67", number)
        match3 = PhoneNumberMatch(10, "1 801 234 45 67", number)

        self.assertEqual(match1, match2)
        self.assertEqual(match1.start, match2.start)
        self.assertEqual(match1.end, match2.end)
        self.assertEqual(match1.number, match2.number)
        self.assertEqual(match1.raw_string, match2.raw_string)
        self.assertEqual("1 800 234 45 67", match1.raw_string)
        # Python-specific: check __ne__()
        self.assertNotEqual(match1, match3)
        self.assertTrue(match1 != match3)
        # Python-specific: Check only comparisons of the same type work
        self.assertNotEqual(match1, None)
        self.assertNotEqual(match1, "")
        self.assertNotEqual(match1, "1 800 234 45 67")
        self.assertNotEqual(match1, 0)

    def testIllegalArguments(self):
        """Tests the value type semantics for matches with a None number."""
        try:
            PhoneNumberMatch(-110, "1 800 234 45 67", PhoneNumber())
            self.fail("Expected failed constructor")
        except Exception:
            pass

        try:
            PhoneNumberMatch(10, "1 800 234 45 67", None)
            self.fail("Expected failed constructor")
        except Exception:
            pass

        try:
            PhoneNumberMatch(10, None, PhoneNumber())
            self.fail("Expected failed constructor")
        except Exception:
            pass

        try:
            PhoneNumberMatch(10, None, None)
            self.fail("Expected failed constructor")
        except Exception:
            pass

    def testStringConvert(self):
        """Check string conversion"""
        number = PhoneNumber()
        match = PhoneNumberMatch(10, "1 800 234 45 67", number)

        self.assertEqual("PhoneNumberMatch [10,25) 1 800 234 45 67", str(match))
        # Python version extra test
        self.assertEqual("PhoneNumberMatch(start=10, raw_string='1 800 234 45 67', "
                         "numobj=PhoneNumber(country_code=None, national_number=None, extension=None, "
                         "italian_leading_zero=None, number_of_leading_zeros=None, "
                         "country_code_source=0, preferred_domestic_carrier_code=None))", repr(match))


class NumberContext(object):
    """Small class that holds the context of the number we are testing
    against. The test will insert the phone number to be found between
    leadingText and trailingText."""
    def __init__(self, leadingText, trailingText):
        self.leadingText = leadingText
        self.trailingText = trailingText


class NumberTest(object):
    """Small class that holds the number we want to test and the region for
    which it should be valid."""
    def __init__(self, rawString, region):
        self.rawString = rawString
        self.region = region

    def __str__(self):
        return "%s (%s)" % (self.rawString, self.region)


# Strings with number-like things that shouldn't be found under any level.
IMPOSSIBLE_CASES = [NumberTest("12345", "US"),
                    NumberTest("23456789", "US"),
                    NumberTest("234567890112", "US"),
                    NumberTest("650+253+1234", "US"),
                    NumberTest("3/10/1984", "CA"),
                    NumberTest("03/27/2011", "US"),
                    NumberTest("31/8/2011", "US"),
                    NumberTest("1/12/2011", "US"),
                    NumberTest("10/12/82", "DE"),
                    NumberTest("650x2531234", "US"),
                    NumberTest("2012-01-02 08:00", "US"),
                    NumberTest("2012/01/02 08:00", "US"),
                    NumberTest("20120102 08:00", "US"),
                    NumberTest("2014-04-12 04:04 PM", "US"),
                    NumberTest("2014-04-12 &nbsp;04:04 PM", "US"),
                    NumberTest("2014-04-12 &nbsp;04:04 PM", "US"),
                    NumberTest("2014-04-12  04:04 PM", "US"),
                    ]

# Strings with number-like things that should only be found under "possible".
POSSIBLE_ONLY_CASES = [NumberTest("7121115678", "US"),  # US numbers cannot start with 7 in the test metadata to be valid.
                       # 'X' should not be found in numbers at leniencies stricter than POSSIBLE, unless it represents
                       # a carrier code or extension.
                       NumberTest("1650 x 253 - 1234", "US"),
                       NumberTest("650 x 253 - 1234", "US"),
                       NumberTest("6502531x234", "US"),
                       NumberTest("(20) 3346 1234", "GB"),  # Non-optional NP omitted
                       ]

# Strings with number-like things that should only be found up to and
# including the "valid" leniency level.
VALID_CASES = [NumberTest("65 02 53 00 00", "US"),
               NumberTest("6502 538365", "US"),
               NumberTest("650//253-1234", "US"),  # 2 slashes are illegal at higher levels
               NumberTest("650/253/1234", "US"),
               NumberTest("9002309. 158", "US"),
               NumberTest("12 7/8 - 14 12/34 - 5", "US"),
               NumberTest("12.1 - 23.71 - 23.45", "US"),
               NumberTest("800 234 1 111x1111", "US"),
               NumberTest("1979-2011 100", "US"),
               NumberTest("+494949-4-94", "DE"),  # National number in wrong format
               NumberTest(u("\uFF14\uFF11\uFF15\uFF16\uFF16\uFF16\uFF16-\uFF17\uFF17\uFF17"), "US"),
               NumberTest("2012-0102 08", "US"),  # Very strange formatting.
               NumberTest("2012-01-02 08", "US"),
               # Breakdown assistance number with unexpected formatting.
               NumberTest("1800-1-0-10 22", "AU"),
               NumberTest("030-3-2 23 12 34", "DE"),
               NumberTest("03 0 -3 2 23 12 34", "DE"),
               NumberTest("(0)3 0 -3 2 23 12 34", "DE"),
               NumberTest("0 3 0 -3 2 23 12 34", "DE"),
               ]

# Strings with number-like things that should only be found up to and
# including the "strict_grouping" leniency level.
STRICT_GROUPING_CASES = [NumberTest("(415) 6667777", "US"),
                         NumberTest("415-6667777", "US"),
                         # Should be found by strict grouping but not exact
                         # grouping, as the last two groups are formatted
                         # together as a block.
                         NumberTest("0800-2491234", "DE"),
                         # Doesn't match any formatting in the test file, but
                         # almost matches an alternate format (the last two
                         # groups have been squashed together here).
                         NumberTest("0900-1 123123", "DE"),
                         NumberTest("(0)900-1 123123", "DE"),
                         NumberTest("0 900-1 123123", "DE"),
                         # NDC also found as part of the country calling code;
                         # this shouldn't ruin the grouping expectations.
                         NumberTest("+33 3 34 2312", "FR"),
                         ]

# Strings with number-like things that should be found at all levels.
EXACT_GROUPING_CASES = [NumberTest(u("\uFF14\uFF11\uFF15\uFF16\uFF16\uFF16\uFF17\uFF17\uFF17\uFF17"), "US"),
                        NumberTest(u("\uFF14\uFF11\uFF15-\uFF16\uFF16\uFF16-\uFF17\uFF17\uFF17\uFF17"), "US"),
                        NumberTest("4156667777", "US"),
                        NumberTest("4156667777 x 123", "US"),
                        NumberTest("415-666-7777", "US"),
                        NumberTest("415/666-7777", "US"),
                        NumberTest("415-666-7777 ext. 503", "US"),
                        NumberTest("1 415 666 7777 x 123", "US"),
                        NumberTest("+1 415-666-7777", "US"),
                        NumberTest("+494949 49", "DE"),
                        NumberTest("+49-49-34", "DE"),
                        NumberTest("+49-4931-49", "DE"),
                        NumberTest("04931-49", "DE"),  # With National Prefix
                        NumberTest("+49-494949", "DE"),  # One group with country code
                        NumberTest("+49-494949 ext. 49", "DE"),
                        NumberTest("+49494949 ext. 49", "DE"),
                        NumberTest("0494949", "DE"),
                        NumberTest("0494949 ext. 49", "DE"),
                        NumberTest("01 (33) 3461 2234", "MX"),  # Optional NP present
                        NumberTest("(33) 3461 2234", "MX"),  # Optional NP omitted
                        NumberTest("1800-10-10 22", "AU"),  # Breakdown assistance number.
                        # Doesn't match any formatting in the test file, but
                        # matches an alternate format exactly.
                        NumberTest("0900-1 123 123", "DE"),
                        NumberTest("(0)900-1 123 123", "DE"),
                        NumberTest("0 900-1 123 123", "DE"),
                        NumberTest("+33 3 34 23 12", "FR"),
                        ]


class PhoneNumberMatcherTest(TestMetadataTestCase):
    """Tests for PhoneNumberMatcher.

    This only tests basic functionality based on test metadata.  See
    testphonenumberutil.py for the origin of the test data.
    """
    def testContainsMoreThanOneSlashInNationalNumber(self):
        # A date should return true.
        number = PhoneNumber(country_code=1,
                             country_code_source=CountryCodeSource.FROM_DEFAULT_COUNTRY)
        candidate = "1/05/2013"
        self.assertTrue(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

        # Here, the country code source thinks it started with a country calling code, but this is not
        # the same as the part before the slash, so it's still true.
        number = PhoneNumber(country_code=274,
                             country_code_source=CountryCodeSource.FROM_NUMBER_WITHOUT_PLUS_SIGN)
        candidate = "27/4/2013"
        self.assertTrue(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

        # Now it should be false, because the first slash is after the country calling code.
        number = PhoneNumber(country_code=49,
                             country_code_source=CountryCodeSource.FROM_NUMBER_WITH_PLUS_SIGN)
        candidate = "49/69/2013"
        self.assertFalse(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

        number = PhoneNumber(country_code=49,
                             country_code_source=CountryCodeSource.FROM_NUMBER_WITHOUT_PLUS_SIGN)
        candidate = "+49/69/2013"
        self.assertFalse(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

        candidate = "+ 49/69/2013"
        self.assertFalse(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

        candidate = "+ 49/69/20/13"
        self.assertTrue(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

        # Here, the first group is not assumed to be the country calling code, even though it is the
        # same as it, so this should return true.
        number = PhoneNumber(country_code=49,
                             country_code_source=CountryCodeSource.FROM_DEFAULT_COUNTRY)
        candidate = "49/69/2013"
        self.assertTrue(phonenumbermatcher._contains_more_than_one_slash_in_national_number(number, candidate))

    # See PhoneNumberUtilTest.testParseNationalNumber().
    def testFindNationalNumber(self):
        # same cases as in testParseNationalNumber
        self.doTestFindInContext("033316005", "NZ")
        # self.doTestFindInContext("33316005", "NZ") is omitted since the
        # national prefix is obligatory for these types of numbers in New Zealand.
        # National prefix attached and some formatting present.
        self.doTestFindInContext("03-331 6005", "NZ")
        self.doTestFindInContext("03 331 6005", "NZ")
        # Testing international prefixes.
        # Should strip country code.
        self.doTestFindInContext("0064 3 331 6005", "NZ")
        # Try again, but this time we have an international number with Region
        # Code US. It should recognize the country code and parse accordingly.
        self.doTestFindInContext("01164 3 331 6005", "US")
        self.doTestFindInContext("+64 3 331 6005", "US")

        self.doTestFindInContext("64(0)64123456", "NZ")
        # Check that using a "/" is fine in a phone number.
        # Note that real Polish numbers do *not* start with a 0.
        self.doTestFindInContext("0123/456789", "PL")
        self.doTestFindInContext("123-456-7890", "US")

    # See PhoneNumberUtilTest.testParseWithInternationalPrefixes().
    def testFindWithInternationalPrefixes(self):
        self.doTestFindInContext("+1 (650) 333-6000", "NZ")
        self.doTestFindInContext("1-650-333-6000", "US")
        # Calling the US number from Singapore by using different service
        # providers
        # 1st test: calling using SingTel IDD service (IDD is 001)
        self.doTestFindInContext("0011-650-333-6000", "SG")
        # 2nd test: calling using StarHub IDD service (IDD is 008)
        self.doTestFindInContext("0081-650-333-6000", "SG")
        # 3rd test: calling using SingTel V019 service (IDD is 019)
        self.doTestFindInContext("0191-650-333-6000", "SG")
        # Calling the US number from Poland
        self.doTestFindInContext("0~01-650-333-6000", "PL")
        # Using "++" at the start.
        self.doTestFindInContext("++1 (650) 333-6000", "PL")
        # Using a full-width plus sign.
        self.doTestFindInContext(u("\uFF0B1 (650) 333-6000"), "SG")
        # The whole number, including punctuation, is here represented in
        # full-width form.
        self.doTestFindInContext(u("\uFF0B\uFF11\u3000\uFF08\uFF16\uFF15\uFF10\uFF09") +
                                 u("\u3000\uFF13\uFF13\uFF13\uFF0D\uFF16\uFF10\uFF10\uFF10"),
                                 "SG")

    # See PhoneNumberUtilTest.testParseWithLeadingZero().
    def testFindWithLeadingZero(self):
        self.doTestFindInContext("+39 02-36618 300", "NZ")
        self.doTestFindInContext("02-36618 300", "IT")
        self.doTestFindInContext("312 345 678", "IT")

    # See PhoneNumberUtilTest.testParseNationalNumberArgentina().
    def testFindNationalNumberArgentina(self):
        # Test parsing mobile numbers of Argentina.
        self.doTestFindInContext("+54 9 343 555 1212", "AR")
        self.doTestFindInContext("0343 15 555 1212", "AR")

        self.doTestFindInContext("+54 9 3715 65 4320", "AR")
        self.doTestFindInContext("03715 15 65 4320", "AR")

        # Test parsing fixed-line numbers of Argentina.
        self.doTestFindInContext("+54 11 3797 0000", "AR")
        self.doTestFindInContext("011 3797 0000", "AR")

        self.doTestFindInContext("+54 3715 65 4321", "AR")
        self.doTestFindInContext("03715 65 4321", "AR")

        self.doTestFindInContext("+54 23 1234 0000", "AR")
        self.doTestFindInContext("023 1234 0000", "AR")

    # See PhoneNumberUtilTest.testParseWithXInNumber().
    def testFindWithXInNumber(self):
        self.doTestFindInContext("(0xx) 123456789", "AR")
        # A case where x denotes both carrier codes and extension symbol.
        self.doTestFindInContext("(0xx) 123456789 x 1234", "AR")

        # This test is intentionally constructed such that the number of digit
        # after xx is larger than 7, so that the number won't be mistakenly
        # treated as an extension, as we allow extensions up to 7 digits. This
        # assumption is okay for now as all the countries where a carrier
        # selection code is written in the form of xx have a national
        # significant number of length larger than 7.
        self.doTestFindInContext("011xx5481429712", "US")

    # See PhoneNumberUtilTest.testParseNumbersMexico().
    def testFindNumbersMexico(self):
        # Test parsing fixed-line numbers of Mexico.
        self.doTestFindInContext("+52 (449)978-0001", "MX")
        self.doTestFindInContext("01 (449)978-0001", "MX")
        self.doTestFindInContext("(449)978-0001", "MX")

        # Test parsing mobile numbers of Mexico.
        self.doTestFindInContext("+52 1 33 1234-5678", "MX")
        self.doTestFindInContext("044 (33) 1234-5678", "MX")
        self.doTestFindInContext("045 33 1234-5678", "MX")

    # See PhoneNumberUtilTest.testParseNumbersWithPlusWithNoRegion().
    def testFindNumbersWithPlusWithNoRegion(self):
        # "ZZ" is allowed only if the number starts with a '+' - then the
        # country code can be calculated.
        self.doTestFindInContext("+64 3 331 6005", "ZZ")
        # None is also allowed for the region code in these cases.
        self.doTestFindInContext("+64 3 331 6005", None)

    # See PhoneNumberUtilTest.testParseExtensions().
    def testFindExtensions(self):
        self.doTestFindInContext("03 331 6005 ext 3456", "NZ")
        self.doTestFindInContext("03-3316005x3456", "NZ")
        self.doTestFindInContext("03-3316005 int.3456", "NZ")
        self.doTestFindInContext("03 3316005 #3456", "NZ")
        self.doTestFindInContext("0~0 1800 7493 524", "PL")
        self.doTestFindInContext("(1800) 7493.524", "US")
        # Check that the last instance of an extension token is matched.
        self.doTestFindInContext("0~0 1800 7493 524 ~1234", "PL")
        # Verifying bug-fix where the last digit of a number was previously omitted if it was a 0 when
        # extracting the extension. Also verifying a few different cases of extensions.
        self.doTestFindInContext("+44 2034567890x456", "NZ")
        self.doTestFindInContext("+44 2034567890x456", "GB")
        self.doTestFindInContext("+44 2034567890 x456", "GB")
        self.doTestFindInContext("+44 2034567890 X456", "GB")
        self.doTestFindInContext("+44 2034567890 X 456", "GB")
        self.doTestFindInContext("+44 2034567890 X    456", "GB")
        self.doTestFindInContext("+44 2034567890    X 456", "GB")

        self.doTestFindInContext("(800) 901-3355 x 7246433", "US")
        self.doTestFindInContext("(800) 901-3355 , ext 7246433", "US")
        self.doTestFindInContext("(800) 901-3355 ,extension 7246433", "US")
        # The next test differs from phonenumberutil -> when matching we don't
        # consider a lone comma to indicate an extension, although we accept
        # it when parsing.
        self.doTestFindInContext("(800) 901-3355 ,x 7246433", "US")
        self.doTestFindInContext("(800) 901-3355 ext: 7246433", "US")

    def testFindInterspersedWithSpace(self):
        self.doTestFindInContext("0 3   3 3 1   6 0 0 5", "NZ")

    # Test matching behavior when starting in the middle of a phone number.
    def testIntermediateParsePositions(self):
        text = "Call 033316005  or 032316005!"
        #       |    |    |    |    |    |
        #       0    5   10   15   20   25

        # Iterate over all possible indices.
        for ii in range(6):
            self.assertEqualRange(text, ii, 5, 14)

        # 7 and 8 digits in a row are still parsed as number.
        self.assertEqualRange(text, 6, 6, 14)
        self.assertEqualRange(text, 7, 7, 14)
        # Anything smaller is skipped to the second instance.
        for ii in range(8, 20):
            self.assertEqualRange(text, ii, 19, 28)

    def testFourMatchesInARow(self):
        number1 = "415-666-7777"
        number2 = "800-443-1223"
        number3 = "212-443-1223"
        number4 = "650-443-1223"
        text = number1 + " - " + number2 + " - " + number3 + " - " + number4

        matcher = PhoneNumberMatcher(text, "US")
        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, text, number1, "US")

        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, text, number2, "US")

        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, text, number3, "US")

        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, text, number4, "US")

    def testMatchesFoundWithMultipleSpaces(self):
        number1 = "(415) 666-7777"
        number2 = "(800) 443-1223"
        text = number1 + " " + number2

        matcher = PhoneNumberMatcher(text, "US")
        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, text, number1, "US")

        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, text, number2, "US")

    def testMatchWithSurroundingZipcodes(self):
        number = "415-666-7777"
        zipPreceding = "My address is CA 34215 - " + number + " is my number."

        matcher = PhoneNumberMatcher(zipPreceding, "US")
        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, zipPreceding, number, "US")

        # Now repeat, but this time the phone number has spaces in it. It should still be found.
        number = "(415) 666 7777"

        zipFollowing = "My number is " + number + ". 34215 is my zip-code."
        matcher = PhoneNumberMatcher(zipFollowing, "US")
        match = matcher.next() if matcher.has_next() else None
        self.assertMatchProperties(match, zipFollowing, number, "US")

    def testIsLatinLetter(self):
        self.assertTrue(PhoneNumberMatcher._is_latin_letter('c'))
        self.assertTrue(PhoneNumberMatcher._is_latin_letter('C'))
        self.assertTrue(PhoneNumberMatcher._is_latin_letter(u("\u00C9")))
        self.assertTrue(PhoneNumberMatcher._is_latin_letter(u("\u0301")))  # Combining acute accent
        # Punctuation, digits and white-space are not considered "latin letters".
        self.assertFalse(PhoneNumberMatcher._is_latin_letter(':'))
        self.assertFalse(PhoneNumberMatcher._is_latin_letter('5'))
        self.assertFalse(PhoneNumberMatcher._is_latin_letter('-'))
        self.assertFalse(PhoneNumberMatcher._is_latin_letter('.'))
        self.assertFalse(PhoneNumberMatcher._is_latin_letter(' '))
        self.assertFalse(PhoneNumberMatcher._is_latin_letter(u("\u6211")))  # Chinese character
        self.assertFalse(PhoneNumberMatcher._is_latin_letter(u("\u306E")))  # Hiragana letter no

    def testMatchesWithSurroundingLatinChars(self):
        possibleOnlyContexts = []
        possibleOnlyContexts.append(NumberContext("abc", "def"))
        possibleOnlyContexts.append(NumberContext("abc", ""))
        possibleOnlyContexts.append(NumberContext("", "def"))
        # Latin capital letter e with an acute accent.
        possibleOnlyContexts.append(NumberContext(u("\u00C9"), ""))
        # e with an acute accent decomposed (with combining mark).
        possibleOnlyContexts.append(NumberContext(u("e\u0301"), ""))

        # Numbers should not be considered valid, if they are surrounded by
        # Latin characters, but should be considered possible.
        self.findMatchesInContexts(possibleOnlyContexts, False, True)

    def testMoneyNotSeenAsPhoneNumber(self):
        possibleOnlyContexts = []
        possibleOnlyContexts.append(NumberContext("$", ""))
        possibleOnlyContexts.append(NumberContext("", "$"))
        possibleOnlyContexts.append(NumberContext(u("\u00A3"), ""))  # Pound sign
        possibleOnlyContexts.append(NumberContext(u("\u00A5"), ""))  # Yen sign
        self.findMatchesInContexts(possibleOnlyContexts, False, True)

    def testPercentageNotSeenAsPhoneNumber(self):
        possibleOnlyContexts = []
        possibleOnlyContexts.append(NumberContext("", "%"))
        # Numbers followed by % should be dropped.
        self.findMatchesInContexts(possibleOnlyContexts, False, True)

    def testPhoneNumberWithLeadingOrTrailingMoneyMatches(self):
        # Because of the space after the 20 (or before the 100) these dollar
        # amounts should not stop the actual number from being found.
        contexts = []
        contexts.append(NumberContext("$20 ", ""))
        contexts.append(NumberContext("", " 100$"))
        self.findMatchesInContexts(contexts, True, True)

    def testMatchesWithSurroundingLatinCharsAndLeadingPunctuation(self):
        # Contexts with trailing characters. Leading characters are okay here
        # since the numbers we will insert start with punctuation, but
        # trailing characters are still not allowed.
        possibleOnlyContexts = []
        possibleOnlyContexts.append(NumberContext("abc", "def"))
        possibleOnlyContexts.append(NumberContext("", "def"))
        possibleOnlyContexts.append(NumberContext("", u("\u00C9")))

        # Numbers should not be considered valid, if they have trailing Latin
        # characters, but should be considered possible.
        numberWithPlus = "+14156667777"
        numberWithBrackets = "(415)6667777"
        self.findMatchesInContexts(possibleOnlyContexts, False, True, "US", numberWithPlus)
        self.findMatchesInContexts(possibleOnlyContexts, False, True, "US", numberWithBrackets)

        validContexts = []
        validContexts.append(NumberContext("abc", ""))
        validContexts.append(NumberContext(u("\u00C9"), ""))
        validContexts.append(NumberContext(u("\u00C9"), "."))  # Trailing punctuation.
        validContexts.append(NumberContext(u("\u00C9"), " def"))  # Trailing white-space.

        # Numbers should be considered valid, since they start with punctuation.
        self.findMatchesInContexts(validContexts, True, True, "US", numberWithPlus)
        self.findMatchesInContexts(validContexts, True, True, "US", numberWithBrackets)

    def testMatchesWithSurroundingChineseChars(self):
        validContexts = []
        validContexts.append(NumberContext(u("\u6211\u7684\u7535\u8BDD\u53F7\u7801\u662F"), ""))
        validContexts.append(NumberContext("", u("\u662F\u6211\u7684\u7535\u8BDD\u53F7\u7801")))
        validContexts.append(NumberContext(u("\u8BF7\u62E8\u6253"), u("\u6211\u5728\u660E\u5929")))

        # Numbers should be considered valid, since they are surrounded by Chinese.
        self.findMatchesInContexts(validContexts, True, True)

    def testMatchesWithSurroundingPunctuation(self):
        validContexts = []
        validContexts.append(NumberContext("My number-", ""))    # At end of text.
        validContexts.append(NumberContext("", ".Nice day."))    # At start of text.
        validContexts.append(NumberContext("Tel:", "."))    # Punctuation surrounds number.
        validContexts.append(NumberContext("Tel: ", " on Saturdays."))    # White-space is also fine.

        # Numbers should be considered valid, since they are surrounded by punctuation.
        self.findMatchesInContexts(validContexts, True, True)

    def testMatchesMultiplePhoneNumbersSeparatedByPhoneNumberPunctuation(self):
        text = "Call 650-253-4561 -- 455-234-3451"
        region = "US"
        number1 = PhoneNumber(country_code=phonenumberutil.country_code_for_region(region),
                              national_number=6502534561)
        match1 = PhoneNumberMatch(5, "650-253-4561", number1)
        number2 = PhoneNumber(country_code=phonenumberutil.country_code_for_region(region),
                              national_number=4552343451)
        match2 = PhoneNumberMatch(21, "455-234-3451", number2)

        matches = PhoneNumberMatcher(text, region)
        self.assertEqual(match1, matches.next())
        self.assertEqual(match2, matches.next())

    def testDoesNotMatchMultiplePhoneNumbersSeparatedWithNoWhiteSpace(self):
        # No white-space found between numbers - neither is found.
        text = "Call 650-253-4561--455-234-3451"
        region = "US"
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher(text, region)))

    def testMatchesWithPossibleLeniency(self):
        testCases = STRICT_GROUPING_CASES + EXACT_GROUPING_CASES + VALID_CASES + POSSIBLE_ONLY_CASES
        self._doTestNumberMatchesForLeniency(testCases, Leniency.POSSIBLE)

    def testNonMatchesWithPossibleLeniency(self):
        testCases = IMPOSSIBLE_CASES
        self._doTestNumberNonMatchesForLeniency(testCases, Leniency.POSSIBLE)

    def testMatchesWithValidLeniency(self):
        testCases = STRICT_GROUPING_CASES + EXACT_GROUPING_CASES + VALID_CASES
        self._doTestNumberMatchesForLeniency(testCases, Leniency.VALID)

    def testNonMatchesWithValidLeniency(self):
        testCases = IMPOSSIBLE_CASES + POSSIBLE_ONLY_CASES
        self._doTestNumberNonMatchesForLeniency(testCases, Leniency.VALID)

    def testMatchesWithStrictGroupingLeniency(self):
        testCases = STRICT_GROUPING_CASES + EXACT_GROUPING_CASES
        self._doTestNumberMatchesForLeniency(testCases, Leniency.STRICT_GROUPING)

    def testNonMatchesWithStrictGroupLeniency(self):
        testCases = IMPOSSIBLE_CASES + POSSIBLE_ONLY_CASES + VALID_CASES
        self._doTestNumberNonMatchesForLeniency(testCases, Leniency.STRICT_GROUPING)

    def testMatchesWithExactGroupingLeniency(self):
        testCases = EXACT_GROUPING_CASES
        self._doTestNumberMatchesForLeniency(testCases, Leniency.EXACT_GROUPING)

    def testNonMatchesExactGroupLeniency(self):
        testCases = IMPOSSIBLE_CASES + POSSIBLE_ONLY_CASES + VALID_CASES + STRICT_GROUPING_CASES
        self._doTestNumberNonMatchesForLeniency(testCases, Leniency.EXACT_GROUPING)

    def _doTestNumberMatchesForLeniency(self, testCases, leniency):
        noMatchFoundCount = 0
        wrongMatchFoundCount = 0
        for test in testCases:
            iterator = self.findNumbersForLeniency(test.rawString, test.region, leniency)
            match = iterator.next() if iterator.has_next() else None
            if match is None:
                noMatchFoundCount += 1
                prnt("No match found in  %s for leniency: %s" % (test, leniency), file=sys.stderr)
            else:
                if test.rawString != match.raw_string:
                    wrongMatchFoundCount += 1
                    prnt("Found wrong match in test %s. Found %s" % (test, match), file=sys.stderr)
        self.assertEqual(0, noMatchFoundCount)
        self.assertEqual(0, wrongMatchFoundCount)

    def _doTestNumberNonMatchesForLeniency(self, testCases, leniency):
        matchFoundCount = 0
        for test in testCases:
            iterator = self.findNumbersForLeniency(test.rawString, test.region, leniency)
            match = iterator.next() if iterator.has_next() else None
            if match is not None:
                matchFoundCount += 1
                prnt("Match found in %s for leniency: %s" % (test, leniency), file=sys.stderr)
        self.assertEqual(0, matchFoundCount)

    def findMatchesInContexts(self, contexts, isValid, isPossible,
                              region="US", number="415-666-7777"):
        """Helper method which tests the contexts provided and ensures
        that:
         - if isValid is True, they all find a test number inserted in the
           middle when leniency of matching is set to VALID; else no test
           number should be extracted at that leniency level
         - if isPossible is True, they all find a test number inserted in the
           middle when leniency of matching is set to POSSIBLE; else no test
           number should be extracted at that leniency level"""
        if isValid:
            self.doTestInContext(number, region, contexts, Leniency.VALID)
        else:
            for context in contexts:
                text = context.leadingText + number + context.trailingText
                self.assertTrue(self.hasNoMatches(PhoneNumberMatcher(text, region)),
                                msg="Should not have found a number in " + text)
        if isPossible:
            self.doTestInContext(number, region, contexts, Leniency.POSSIBLE)
        else:
            for context in contexts:
                text = context.leadingText + number + context.trailingText
                self.assertTrue(self.hasNoMatches(PhoneNumberMatcher(text, region,
                                                                     leniency=Leniency.POSSIBLE, max_tries=65535)),
                                msg="Should not have found a number in " + text)

    def testNonMatchingBracketsAreInvalid(self):
        # The digits up to the ", " form a valid US number, but it shouldn't
        # be matched as one since there was a non-matching bracket present.
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("80.585 [79.964, 81.191]", "US")))

        # The trailing "]" is thrown away before parsing, so the resultant
        # number, while a valid US number, does not have matching brackets.
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("80.585 [79.964]", "US")))

        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("80.585 ((79.964)", "US")))

        # This case has too many sets of brackets to be valid.
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("(80).(585) (79).(9)64", "US")))

    def testNoMatchIfRegionIsNone(self):
        # Fail on non-international prefix if region code is None.
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("Random text body - number is 0331 6005, see you there", None)))

    def testNoMatchInEmptyString(self):
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("", "US")))
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("  ", "US")))

    def testNoMatchIfNoNumber(self):
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher("Random text body - number is foobar, see you there", "US")))

    def testSequences(self):
        # Test multiple occurrences.
        text = "Call 033316005  or 032316005!"
        region = "NZ"

        number1 = PhoneNumber()
        number1.country_code = phonenumberutil.country_code_for_region(region)
        number1.national_number = 33316005
        match1 = PhoneNumberMatch(5, "033316005", number1)

        number2 = PhoneNumber()
        number2.country_code = phonenumberutil.country_code_for_region(region)
        number2.national_number = 32316005
        match2 = PhoneNumberMatch(19, "032316005", number2)

        matcher = PhoneNumberMatcher(text, region, Leniency.POSSIBLE, 65535)

        self.assertEqual(match1, matcher.next())
        self.assertEqual(match2, matcher.next())
        self.assertFalse(matcher.has_next())

    def testNoneInput(self):
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher(None, "US")))
        self.assertTrue(self.hasNoMatches(PhoneNumberMatcher(None, None)))

    def testMaxMatches(self):
        # Set up text with 100 valid phone numbers.
        numbers = "My info: 415-666-7777," * 100

        # Matches all 100. Max only applies to failed cases.
        number = phonenumberutil.parse("+14156667777", None)
        expected = [number] * 100

        matcher = PhoneNumberMatcher(numbers, "US", Leniency.VALID, 10)
        actual = [x.number for x in matcher]

        self.assertEqual(expected, actual)

    def testMaxMatchesInvalid(self):
        # Set up text with 10 invalid phone numbers followed by 100 valid.
        numbers = (("My address 949-8945-0" * 10) +
                   ("My info: 415-666-7777," * 100))
        matcher = PhoneNumberMatcher(numbers, "US", Leniency.VALID, 10)
        self.assertFalse(matcher.has_next())

    def testMaxMatchesMixed(self):
        # Set up text with 100 valid numbers inside an invalid number.
        numbers = "My info: 415-666-7777 123 fake street" * 100

        # Only matches the first 10 despite there being 100 numbers due to max matches.
        number = phonenumberutil.parse("+14156667777", None)
        expected = [number] * 10

        matcher = PhoneNumberMatcher(numbers, "US", Leniency.VALID, 10)
        actual = [x.number for x in matcher]

        self.assertEqual(expected, actual)

    def testNonPlusPrefixedNumbersNotFoundForInvalidRegion(self):
        # Does not start with a "+", we won't match it.
        matcher = PhoneNumberMatcher("1 456 764 156", "ZZ")
        self.assertFalse(matcher.has_next())
        try:
            matcher.next()
            self.fail("Violation of the Iterator contract.")
        except Exception:
            # Success
            pass
        self.assertFalse(matcher.has_next())

    def testEmptyIteration(self):
        matcher = PhoneNumberMatcher("", "ZZ")
        self.assertFalse(matcher.has_next())
        self.assertFalse(matcher.has_next())
        try:
            matcher.next()
            self.fail("Violation of the iterator contract.")
        except Exception:
            # Success
            pass
        self.assertFalse(matcher.has_next())

    def testSingleIteration(self):
        matcher = PhoneNumberMatcher("+14156667777", "ZZ")

        # With hasNext() -> next().
        # Double hasNext() to ensure it does not advance.
        self.assertTrue(matcher.has_next())
        self.assertTrue(matcher.has_next())
        self.assertTrue(matcher.next() is not None)
        self.assertFalse(matcher.has_next())
        try:
            matcher.next()
            self.fail("Violation of the Matcher contract.")
        except Exception:
            # Success
            pass
        self.assertFalse(matcher.has_next())

        # With next() only.
        matcher = PhoneNumberMatcher("+14156667777", "ZZ")
        self.assertTrue(matcher.next() is not None)
        try:
            matcher.next()
            self.fail("Violation of the Matcher contract.")
        except Exception:
            # Success
            pass

    def testDoubleIteration(self):
        matcher = PhoneNumberMatcher("+14156667777 foobar +14156667777 ", "ZZ")

        # With hasNext() -> next().
        # Double hasNext() to ensure it does not advance.
        self.assertTrue(matcher.has_next())
        self.assertTrue(matcher.has_next())
        self.assertTrue(matcher.next() is not None)
        self.assertTrue(matcher.has_next())
        self.assertTrue(matcher.has_next())
        self.assertTrue(matcher.next() is not None)
        self.assertFalse(matcher.has_next())
        try:
            matcher.next()
            self.fail("Violation of the Matcher contract.")
        except Exception:
            # Success
            pass
        self.assertFalse(matcher.has_next())

        # With next() only.
        matcher = PhoneNumberMatcher("+14156667777 foobar +14156667777 ", "ZZ")
        self.assertTrue(matcher.next() is not None)
        self.assertTrue(matcher.next() is not None)
        try:
            matcher.next()
            self.fail("Violation of the Matcher contract.")
        except Exception:
            # Success
            pass

    def assertEqualRange(self, text, index, start, end):
        """Asserts that another number can be found in text starting at index, and that
        its corresponding range is [start, end).
        """
        sub = text[index:]
        matcher = PhoneNumberMatcher(sub, "NZ", Leniency.POSSIBLE, 65535)

        self.assertTrue(matcher.has_next())
        match = matcher.next()
        self.assertEqual(start - index, match.start)
        self.assertEqual(end - index, match.end)
        self.assertEqual(sub[match.start:match.end], match.raw_string)

    def assertMatchProperties(self, match, text, number, region):
        """Asserts that the expected match is non-null, and that the raw string
        and expected proto buffer are set appropriately."""
        expectedResult = phonenumberutil.parse(number, region)
        self.assertTrue(match is not None,
                        msg="Did not find a number in '" + text + "'; expected " + number)
        self.assertEqual(expectedResult, match.number)
        self.assertEqual(number, match.raw_string)

    def doTestFindInContext(self, number, defaultCountry):
        """Tests numbers found by PhoneNumberMatcher in various textual contexts"""
        self.findPossibleInContext(number, defaultCountry)
        parsed = phonenumberutil.parse(number, defaultCountry)
        if phonenumberutil.is_valid_number(parsed):
            self.findValidInContext(number, defaultCountry)

    def findPossibleInContext(self, number, defaultCountry):
        """Tests valid numbers in contexts that should pass for Leniency.POSSIBLE"""
        contextPairs = [NumberContext("", ""),    # no context
                        NumberContext("     ", "\t"),    # whitespace only
                        NumberContext("Hello ", ""),    # no context at end
                        NumberContext("", " to call me!"),    # no context at start
                        NumberContext("Hi there, call ", " to reach me!"),    # no context at start
                        NumberContext("Hi there, call ", ", or don't"),    # with commas
                        # Three examples without whitespace around the number.
                        NumberContext("Hi call", ""),
                        NumberContext("", "forme"),
                        NumberContext("Hi call", "forme"),
                        # With other small numbers.
                        NumberContext("It's cheap! Call ", " before 6:30"),
                        # With a second number later.
                        NumberContext("Call ", " or +1800-123-4567!"),
                        NumberContext("Call me on June 2 at", ""),    # with a Month-Day date
                        # With publication pages.
                        NumberContext("As quoted by Alfonso 12-15 (2009), you may call me at ", ""),
                        NumberContext("As quoted by Alfonso et al. 12-15 (2009), you may call me at ", ""),
                        # With dates, written in the American style.
                        NumberContext("As I said on 03/10/2011, you may call me at ", ""),
                        # With trailing numbers after a comma. The 45 should not be considered an extension.
                        NumberContext("", ", 45 days a year"),
                        # When matching we don't consider semicolon along with legitimate extension
                        # symbol to indicate an extension. The 7246433 should not be considered an
                        # extension.
                        NumberContext("", ";x 7246433"),
                        # With a postfix stripped off as it looks like the start of another number.
                        NumberContext("Call ", "/x12 more"),
                        ]

        self.doTestInContext(number, defaultCountry, contextPairs, Leniency.POSSIBLE)

    def findValidInContext(self, number, defaultCountry):
        """Tests valid numbers in contexts that fail for Leniency.POSSIBLE but
        are valid for Leniency.VALID."""
        contextPairs = [
            # With other small numbers.
            NumberContext("It's only 9.99! Call ", " to buy"),
            # With a number Day.Month.Year date.
            NumberContext("Call me on 21.6.1984 at ", ""),
            # With a number Month/Day date.
            NumberContext("Call me on 06/21 at ", ""),
            # With a number Day.Month date.
            NumberContext("Call me on 21.6. at ", ""),
            # With a number Month/Day/Year date.
            NumberContext("Call me on 06/21/84 at ", ""),
        ]
        self.doTestInContext(number, defaultCountry, contextPairs, Leniency.VALID)

    def doTestInContext(self, number, defaultCountry, contextPairs, leniency):
        for context in contextPairs:
            prefix = context.leadingText
            text = prefix + number + context.trailingText

            start = len(prefix)
            end = start + len(number)
            matcher = PhoneNumberMatcher(text, defaultCountry, leniency, 65535)

            match = matcher.next() if matcher.has_next() else None
            self.assertTrue(match is not None,
                            msg="Did not find a number in '" + text + "'; expected '" + number + "'")

            extracted = text[match.start:match.end]
            self.assertEqual(start, match.start,
                             msg="Unexpected phone region in '" + text + "'; extracted '" + extracted + "'")
            self.assertEqual(end, match.end,
                             msg="Unexpected phone region in '" + text + "'; extracted '" + extracted + "'")
            self.assertEqual(number, extracted)
            self.assertEqual(match.raw_string, extracted)

            self.ensureTermination(text, defaultCountry, leniency)

    # Exhaustively searches for phone numbers from each index within text to
    # test that finding matches always terminates.
    def ensureTermination(self, text, defaultCountry, leniency):
        for index in range(len(text) + 1):
            sub = text[index:]
            matches = ""
            # Iterates over all matches.
            for match in PhoneNumberMatcher(sub, defaultCountry, leniency, 65535):
                matches += ", " + str(match)

    def findNumbersForLeniency(self, text, defaultCountry, leniency):
        return PhoneNumberMatcher(text, defaultCountry, leniency, 65535)

    def hasNoMatches(self, matcher):
        """Returns True if there were no matches found."""
        return not matcher.has_next()

    def testDoubleExtensionX(self):
        # Python version extra test - multiple x for extension marker
        xx_ext = "800 234 1 111 xx 1111"
        # This gives different results for different leniency values (and so
        # can't be used in a NumberTest).
        m0 = PhoneNumberMatcher(xx_ext, "US", leniency=Leniency.POSSIBLE).next()
        self.assertEqual(xx_ext, m0.raw_string)
        matcher2 = PhoneNumberMatcher(xx_ext, "US", leniency=Leniency.STRICT_GROUPING)
        self.assertFalse(matcher2.has_next())

    def testInternals(self):
        # Python-specific test: coverage of internals
        from phonenumbers.phonenumbermatcher import _limit, _verify, _is_national_prefix_present_if_required, _get_national_number_groups
        from phonenumbers import CountryCodeSource
        self.assertEqual("{1,2}", _limit(1, 2))
        self.assertRaises(Exception, _limit, *(-1, 2))
        self.assertRaises(Exception, _limit, *(1, 0))
        self.assertRaises(Exception, _limit, *(2, 1))
        number = PhoneNumber(country_code=44, national_number=7912345678)
        self.assertRaises(Exception, _verify, *(99, number, "12345678"))
        self.assertRaises(ValueError, PhoneNumberMatcher, *("text", "US"), **{"leniency": None})
        self.assertRaises(ValueError, PhoneNumberMatcher, *("text", "US"), **{"max_tries": -2})
        # Invalid country looks like national prefix is present (no way to tell)
        number2 = PhoneNumber(country_code=99, national_number=12345678, country_code_source=CountryCodeSource.FROM_DEFAULT_COUNTRY)
        self.assertTrue(_is_national_prefix_present_if_required(number2))
        # National prefix rule has no lead digits
        number3 = PhoneNumber(country_code=61, national_number=1234567890, country_code_source=CountryCodeSource.FROM_DEFAULT_COUNTRY)
        self.assertTrue(_is_national_prefix_present_if_required(number3))
        # Coverage for _get_national_number_groups() with a formatting pattern provided
        us_number = PhoneNumber(country_code=1, national_number=6502530000)
        num_format = NumberFormat(pattern="(\\d{3})(\\d{3})(\\d{4})", format="\\1-\\2-\\3")
        self.assertEqual(["650", "253", "0000"],
                         _get_national_number_groups(us_number, num_format))



#!/usr/bin/env python
"""Unit tests for asyoutypeformatter.py"""

# Based on original Java code:
#     java/test/com/google/i18n/phonenumbers/PhoneNumberTest.java
# Copyright (C) 2009 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from phonenumbers import AsYouTypeFormatter
from phonenumbers import PhoneMetadata, PhoneNumberDesc, NumberFormat
# Access internal functions of phonenumberutil.py
from phonenumbers import phonenumberutil
from phonenumbers.util import u
from .testmetadatatest import TestMetadataTestCase


class AsYouTypeFormatterTest(TestMetadataTestCase):
    """Unit tests for AsYouTypeFormatter.java

    Note that these tests use the test metadata, not the normal metadata file,
    so should not be used for regression test purposes - these tests are
    illustrative only and test functionality.
    """

    def testInvalidRegion(self):
        formatter = AsYouTypeFormatter("ZZ")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+4", formatter.input_digit('4'))
        self.assertEqual("+48 ", formatter.input_digit('8'))
        self.assertEqual("+48 8", formatter.input_digit('8'))
        self.assertEqual("+48 88", formatter.input_digit('8'))
        self.assertEqual("+48 88 1", formatter.input_digit('1'))
        self.assertEqual("+48 88 12", formatter.input_digit('2'))
        self.assertEqual("+48 88 123", formatter.input_digit('3'))
        self.assertEqual("+48 88 123 1", formatter.input_digit('1'))
        self.assertEqual("+48 88 123 12", formatter.input_digit('2'))

        formatter.clear()
        self.assertEqual("6", formatter.input_digit('6'))
        self.assertEqual("65", formatter.input_digit('5'))
        self.assertEqual("650", formatter.input_digit('0'))
        self.assertEqual("6502", formatter.input_digit('2'))
        self.assertEqual("65025", formatter.input_digit('5'))
        self.assertEqual("650253", formatter.input_digit('3'))

    def testInvalidPlusSign(self):
        formatter = AsYouTypeFormatter("ZZ")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+4", formatter.input_digit('4'))
        self.assertEqual("+48 ", formatter.input_digit('8'))
        self.assertEqual("+48 8", formatter.input_digit('8'))
        self.assertEqual("+48 88", formatter.input_digit('8'))
        self.assertEqual("+48 88 1", formatter.input_digit('1'))
        self.assertEqual("+48 88 12", formatter.input_digit('2'))
        self.assertEqual("+48 88 123", formatter.input_digit('3'))
        self.assertEqual("+48 88 123 1", formatter.input_digit('1'))
        # A plus sign can only appear at the beginning of the number;
        # otherwise, no formatting is applied.
        self.assertEqual("+48881231+", formatter.input_digit('+'))
        self.assertEqual("+48881231+2", formatter.input_digit('2'))

    def testTooLongNumberMatchingMultipleLeadingDigits(self):
        # See https://github.com/googlei18n/libphonenumber/issues/36
        # The bug occurred last time for countries which have two
        # formatting rules with exactly the same leading digits pattern
        # but differ in length.
        formatter = AsYouTypeFormatter("ZZ")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+81 ", formatter.input_digit('1'))
        self.assertEqual("+81 9", formatter.input_digit('9'))
        self.assertEqual("+81 90", formatter.input_digit('0'))
        self.assertEqual("+81 90 1", formatter.input_digit('1'))
        self.assertEqual("+81 90 12", formatter.input_digit('2'))
        self.assertEqual("+81 90 123", formatter.input_digit('3'))
        self.assertEqual("+81 90 1234", formatter.input_digit('4'))
        self.assertEqual("+81 90 1234 5", formatter.input_digit('5'))
        self.assertEqual("+81 90 1234 56", formatter.input_digit('6'))
        self.assertEqual("+81 90 1234 567", formatter.input_digit('7'))
        self.assertEqual("+81 90 1234 5678", formatter.input_digit('8'))
        self.assertEqual("+81 90 12 345 6789", formatter.input_digit('9'))
        self.assertEqual("+81901234567890", formatter.input_digit('0'))
        self.assertEqual("+819012345678901", formatter.input_digit('1'))

    def testCountryWithSpaceInNationalPrefixFormattingRule(self):
        formatter = AsYouTypeFormatter("BY")
        self.assertEqual("8", formatter.input_digit('8'))
        self.assertEqual("88", formatter.input_digit('8'))
        self.assertEqual("881", formatter.input_digit('1'))
        self.assertEqual("8 819", formatter.input_digit('9'))
        self.assertEqual("8 8190", formatter.input_digit('0'))
        # The formatting rule for 5 digit numbers states that no space should
        # be present after the national prefix.
        self.assertEqual("881 901", formatter.input_digit('1'))
        self.assertEqual("8 819 012", formatter.input_digit('2'))
        # Too long, no formatting rule applies.
        self.assertEqual("88190123", formatter.input_digit('3'))

    def testCountryWithSpaceInNationalPrefixFormattingRuleAndLongNdd(self):
        formatter = AsYouTypeFormatter("BY")
        self.assertEqual("9", formatter.input_digit('9'))
        self.assertEqual("99", formatter.input_digit('9'))
        self.assertEqual("999", formatter.input_digit('9'))
        self.assertEqual("9999", formatter.input_digit('9'))
        self.assertEqual("99999 ", formatter.input_digit('9'))
        self.assertEqual("99999 1", formatter.input_digit('1'))
        self.assertEqual("99999 12", formatter.input_digit('2'))
        self.assertEqual("99999 123", formatter.input_digit('3'))
        self.assertEqual("99999 1234", formatter.input_digit('4'))
        self.assertEqual("99999 12 345", formatter.input_digit('5'))

    def testAYTFUS(self):
        formatter = AsYouTypeFormatter("US")
        self.assertEqual("6", formatter.input_digit('6'))
        self.assertEqual("65", formatter.input_digit('5'))
        self.assertEqual("650", formatter.input_digit('0'))
        self.assertEqual("650 2", formatter.input_digit('2'))
        self.assertEqual("650 25", formatter.input_digit('5'))
        self.assertEqual("650 253", formatter.input_digit('3'))
        # Note this is how a US local number (without area code) should be formatted.
        self.assertEqual("650 2532", formatter.input_digit('2'))
        self.assertEqual("650 253 22", formatter.input_digit('2'))
        self.assertEqual("650 253 222", formatter.input_digit('2'))
        self.assertEqual("650 253 2222", formatter.input_digit('2'))

        formatter.clear()
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("16", formatter.input_digit('6'))
        self.assertEqual("1 65", formatter.input_digit('5'))
        self.assertEqual("1 650", formatter.input_digit('0'))
        self.assertEqual("1 650 2", formatter.input_digit('2'))
        self.assertEqual("1 650 25", formatter.input_digit('5'))
        self.assertEqual("1 650 253", formatter.input_digit('3'))
        self.assertEqual("1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("1 650 253 22", formatter.input_digit('2'))
        self.assertEqual("1 650 253 222", formatter.input_digit('2'))
        self.assertEqual("1 650 253 2222", formatter.input_digit('2'))

        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011 ", formatter.input_digit('1'))
        self.assertEqual("011 4", formatter.input_digit('4'))
        self.assertEqual("011 44 ", formatter.input_digit('4'))
        self.assertEqual("011 44 6", formatter.input_digit('6'))
        self.assertEqual("011 44 61", formatter.input_digit('1'))
        self.assertEqual("011 44 6 12", formatter.input_digit('2'))
        self.assertEqual("011 44 6 123", formatter.input_digit('3'))
        self.assertEqual("011 44 6 123 1", formatter.input_digit('1'))
        self.assertEqual("011 44 6 123 12", formatter.input_digit('2'))
        self.assertEqual("011 44 6 123 123", formatter.input_digit('3'))
        self.assertEqual("011 44 6 123 123 1", formatter.input_digit('1'))
        self.assertEqual("011 44 6 123 123 12", formatter.input_digit('2'))
        self.assertEqual("011 44 6 123 123 123", formatter.input_digit('3'))

        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011 ", formatter.input_digit('1'))
        self.assertEqual("011 5", formatter.input_digit('5'))
        self.assertEqual("011 54 ", formatter.input_digit('4'))
        self.assertEqual("011 54 9", formatter.input_digit('9'))
        self.assertEqual("011 54 91", formatter.input_digit('1'))
        self.assertEqual("011 54 9 11", formatter.input_digit('1'))
        self.assertEqual("011 54 9 11 2", formatter.input_digit('2'))
        self.assertEqual("011 54 9 11 23", formatter.input_digit('3'))
        self.assertEqual("011 54 9 11 231", formatter.input_digit('1'))
        self.assertEqual("011 54 9 11 2312", formatter.input_digit('2'))
        self.assertEqual("011 54 9 11 2312 1", formatter.input_digit('1'))
        self.assertEqual("011 54 9 11 2312 12", formatter.input_digit('2'))
        self.assertEqual("011 54 9 11 2312 123", formatter.input_digit('3'))
        self.assertEqual("011 54 9 11 2312 1234", formatter.input_digit('4'))

        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011 ", formatter.input_digit('1'))
        self.assertEqual("011 2", formatter.input_digit('2'))
        self.assertEqual("011 24", formatter.input_digit('4'))
        self.assertEqual("011 244 ", formatter.input_digit('4'))
        self.assertEqual("011 244 2", formatter.input_digit('2'))
        self.assertEqual("011 244 28", formatter.input_digit('8'))
        self.assertEqual("011 244 280", formatter.input_digit('0'))
        self.assertEqual("011 244 280 0", formatter.input_digit('0'))
        self.assertEqual("011 244 280 00", formatter.input_digit('0'))
        self.assertEqual("011 244 280 000", formatter.input_digit('0'))
        self.assertEqual("011 244 280 000 0", formatter.input_digit('0'))
        self.assertEqual("011 244 280 000 00", formatter.input_digit('0'))
        self.assertEqual("011 244 280 000 000", formatter.input_digit('0'))

        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+4", formatter.input_digit('4'))
        self.assertEqual("+48 ", formatter.input_digit('8'))
        self.assertEqual("+48 8", formatter.input_digit('8'))
        self.assertEqual("+48 88", formatter.input_digit('8'))
        self.assertEqual("+48 88 1", formatter.input_digit('1'))
        self.assertEqual("+48 88 12", formatter.input_digit('2'))
        self.assertEqual("+48 88 123", formatter.input_digit('3'))
        self.assertEqual("+48 88 123 1", formatter.input_digit('1'))
        self.assertEqual("+48 88 123 12", formatter.input_digit('2'))
        self.assertEqual("+48 88 123 12 1", formatter.input_digit('1'))
        self.assertEqual("+48 88 123 12 12", formatter.input_digit('2'))

        # Python version extra test: invalid country code after IDD
        formatter.clear()
        self.assertEqual('0', formatter.input_digit('0'))
        self.assertEqual('01', formatter.input_digit('1'))
        self.assertEqual('011 ', formatter.input_digit('1'))
        self.assertEqual('011 4', formatter.input_digit('4'))
        self.assertEqual('011 42', formatter.input_digit('2'))
        self.assertEqual('011 422', formatter.input_digit('2'))
        self.assertEqual('011 4221', formatter.input_digit('1'))
        self.assertEqual('011 42212', formatter.input_digit('2'))
        self.assertEqual('011 422123', formatter.input_digit('3'))

    def testAYTFUSFullWidthCharacters(self):
        formatter = AsYouTypeFormatter("US")
        self.assertEqual(u("\uFF16"), formatter.input_digit(u("\uFF16")))
        self.assertEqual(u("\uFF16\uFF15"), formatter.input_digit(u("\uFF15")))
        self.assertEqual("650", formatter.input_digit(u("\uFF10")))
        self.assertEqual("650 2", formatter.input_digit(u("\uFF12")))
        self.assertEqual("650 25", formatter.input_digit(u("\uFF15")))
        self.assertEqual("650 253", formatter.input_digit(u("\uFF13")))
        self.assertEqual("650 2532", formatter.input_digit(u("\uFF12")))
        self.assertEqual("650 253 22", formatter.input_digit(u("\uFF12")))
        self.assertEqual("650 253 222", formatter.input_digit(u("\uFF12")))
        self.assertEqual("650 253 2222", formatter.input_digit(u("\uFF12")))

    def testAYTFUSMobileShortCode(self):
        formatter = AsYouTypeFormatter("US")
        self.assertEqual("*", formatter.input_digit('*'))
        self.assertEqual("*1", formatter.input_digit('1'))
        self.assertEqual("*12", formatter.input_digit('2'))
        self.assertEqual("*121", formatter.input_digit('1'))
        self.assertEqual("*121#", formatter.input_digit('#'))

    def testAYTFUSVanityNumber(self):
        formatter = AsYouTypeFormatter("US")
        self.assertEqual("8", formatter.input_digit('8'))
        self.assertEqual("80", formatter.input_digit('0'))
        self.assertEqual("800", formatter.input_digit('0'))
        self.assertEqual("800 ", formatter.input_digit(' '))
        self.assertEqual("800 M", formatter.input_digit('M'))
        self.assertEqual("800 MY", formatter.input_digit('Y'))
        self.assertEqual("800 MY ", formatter.input_digit(' '))
        self.assertEqual("800 MY A", formatter.input_digit('A'))
        self.assertEqual("800 MY AP", formatter.input_digit('P'))
        self.assertEqual("800 MY APP", formatter.input_digit('P'))
        self.assertEqual("800 MY APPL", formatter.input_digit('L'))
        self.assertEqual("800 MY APPLE", formatter.input_digit('E'))

    def testAYTFAndRememberPositionUS(self):
        formatter = AsYouTypeFormatter("US")
        self.assertEqual("1", formatter.input_digit('1', remember_position=True))
        self.assertEqual(1, formatter.get_remembered_position())
        self.assertEqual("16", formatter.input_digit('6'))
        self.assertEqual("1 65", formatter.input_digit('5'))
        self.assertEqual(1, formatter.get_remembered_position())
        self.assertEqual("1 650", formatter.input_digit('0', remember_position=True))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("1 650 2", formatter.input_digit('2'))
        self.assertEqual("1 650 25", formatter.input_digit('5'))
        # Note the remembered position for digit "0" changes from 4 to 5, because a space is now
        # inserted in the front.
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("1 650 253", formatter.input_digit('3'))
        self.assertEqual("1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("1 650 253 22", formatter.input_digit('2'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("1 650 253 222", formatter.input_digit('2', remember_position=True))
        self.assertEqual(13, formatter.get_remembered_position())
        self.assertEqual("1 650 253 2222", formatter.input_digit('2'))
        self.assertEqual(13, formatter.get_remembered_position())
        self.assertEqual("165025322222", formatter.input_digit('2'))
        self.assertEqual(10, formatter.get_remembered_position())
        self.assertEqual("1650253222222", formatter.input_digit('2'))
        self.assertEqual(10, formatter.get_remembered_position())

        formatter.clear()
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("16", formatter.input_digit('6', remember_position=True))
        self.assertEqual(2, formatter.get_remembered_position())
        self.assertEqual("1 65", formatter.input_digit('5'))
        self.assertEqual("1 650", formatter.input_digit('0'))
        self.assertEqual(3, formatter.get_remembered_position())
        self.assertEqual("1 650 2", formatter.input_digit('2'))
        self.assertEqual("1 650 25", formatter.input_digit('5'))
        self.assertEqual(3, formatter.get_remembered_position())
        self.assertEqual("1 650 253", formatter.input_digit('3'))
        self.assertEqual("1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("1 650 253 22", formatter.input_digit('2'))
        self.assertEqual(3, formatter.get_remembered_position())
        self.assertEqual("1 650 253 222", formatter.input_digit('2'))
        self.assertEqual("1 650 253 2222", formatter.input_digit('2'))
        self.assertEqual("165025322222", formatter.input_digit('2'))
        self.assertEqual(2, formatter.get_remembered_position())
        self.assertEqual("1650253222222", formatter.input_digit('2'))
        self.assertEqual(2, formatter.get_remembered_position())

        formatter.clear()
        self.assertEqual("6", formatter.input_digit('6'))
        self.assertEqual("65", formatter.input_digit('5'))
        self.assertEqual("650", formatter.input_digit('0'))
        self.assertEqual("650 2", formatter.input_digit('2'))
        self.assertEqual("650 25", formatter.input_digit('5'))
        self.assertEqual("650 253", formatter.input_digit('3'))
        self.assertEqual("650 2532", formatter.input_digit('2', remember_position=True))
        self.assertEqual(8, formatter.get_remembered_position())
        self.assertEqual("650 253 22", formatter.input_digit('2'))
        self.assertEqual(9, formatter.get_remembered_position())
        self.assertEqual("650 253 222", formatter.input_digit('2'))
        # No more formatting when semicolon is entered.
        self.assertEqual("650253222;", formatter.input_digit(';'))
        self.assertEqual(7, formatter.get_remembered_position())
        self.assertEqual("650253222;2", formatter.input_digit('2'))

        formatter.clear()
        self.assertEqual("6", formatter.input_digit('6'))
        self.assertEqual("65", formatter.input_digit('5'))
        self.assertEqual("650", formatter.input_digit('0'))
        # No more formatting when users choose to do their own formatting.
        self.assertEqual("650-", formatter.input_digit('-'))
        self.assertEqual("650-2", formatter.input_digit('2', remember_position=True))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("650-25", formatter.input_digit('5'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("650-253", formatter.input_digit('3'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("650-253-", formatter.input_digit('-'))
        self.assertEqual("650-253-2", formatter.input_digit('2'))
        self.assertEqual("650-253-22", formatter.input_digit('2'))
        self.assertEqual("650-253-222", formatter.input_digit('2'))
        self.assertEqual("650-253-2222", formatter.input_digit('2'))

        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011 ", formatter.input_digit('1'))
        self.assertEqual("011 4", formatter.input_digit('4', remember_position=True))
        self.assertEqual("011 48 ", formatter.input_digit('8'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("011 48 8", formatter.input_digit('8'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("011 48 88", formatter.input_digit('8'))
        self.assertEqual("011 48 88 1", formatter.input_digit('1'))
        self.assertEqual("011 48 88 12", formatter.input_digit('2'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("011 48 88 123", formatter.input_digit('3'))
        self.assertEqual("011 48 88 123 1", formatter.input_digit('1'))
        self.assertEqual("011 48 88 123 12", formatter.input_digit('2'))
        self.assertEqual("011 48 88 123 12 1", formatter.input_digit('1'))
        self.assertEqual("011 48 88 123 12 12", formatter.input_digit('2'))

        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+1", formatter.input_digit('1'))
        self.assertEqual("+1 6", formatter.input_digit('6', remember_position=True))
        self.assertEqual("+1 65", formatter.input_digit('5'))
        self.assertEqual("+1 650", formatter.input_digit('0'))
        self.assertEqual(4, formatter.get_remembered_position())
        self.assertEqual("+1 650 2", formatter.input_digit('2'))
        self.assertEqual(4, formatter.get_remembered_position())
        self.assertEqual("+1 650 25", formatter.input_digit('5'))
        self.assertEqual("+1 650 253", formatter.input_digit('3', remember_position=True))
        self.assertEqual("+1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("+1 650 253 22", formatter.input_digit('2'))
        self.assertEqual("+1 650 253 222", formatter.input_digit('2'))
        self.assertEqual(10, formatter.get_remembered_position())

        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+1", formatter.input_digit('1'))
        self.assertEqual("+1 6", formatter.input_digit('6', remember_position=True))
        self.assertEqual("+1 65", formatter.input_digit('5'))
        self.assertEqual("+1 650", formatter.input_digit('0'))
        self.assertEqual(4, formatter.get_remembered_position())
        self.assertEqual("+1 650 2", formatter.input_digit('2'))
        self.assertEqual(4, formatter.get_remembered_position())
        self.assertEqual("+1 650 25", formatter.input_digit('5'))
        self.assertEqual("+1 650 253", formatter.input_digit('3'))
        self.assertEqual("+1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("+1 650 253 22", formatter.input_digit('2'))
        self.assertEqual("+1 650 253 222", formatter.input_digit('2'))
        self.assertEqual("+1650253222;", formatter.input_digit(';'))
        self.assertEqual(3, formatter.get_remembered_position())

    def testAYTFGBFixedLine(self):
        formatter = AsYouTypeFormatter("GB")
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("02", formatter.input_digit('2'))
        self.assertEqual("020", formatter.input_digit('0'))
        self.assertEqual("020 7", formatter.input_digit('7', remember_position=True))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("020 70", formatter.input_digit('0'))
        self.assertEqual("020 703", formatter.input_digit('3'))
        self.assertEqual(5, formatter.get_remembered_position())
        self.assertEqual("020 7031", formatter.input_digit('1'))
        self.assertEqual("020 7031 3", formatter.input_digit('3'))
        self.assertEqual("020 7031 30", formatter.input_digit('0'))
        self.assertEqual("020 7031 300", formatter.input_digit('0'))
        self.assertEqual("020 7031 3000", formatter.input_digit('0'))

    def testAYTFGBTollFree(self):
        formatter = AsYouTypeFormatter("GB")
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("08", formatter.input_digit('8'))
        self.assertEqual("080", formatter.input_digit('0'))
        self.assertEqual("080 7", formatter.input_digit('7'))
        self.assertEqual("080 70", formatter.input_digit('0'))
        self.assertEqual("080 703", formatter.input_digit('3'))
        self.assertEqual("080 7031", formatter.input_digit('1'))
        self.assertEqual("080 7031 3", formatter.input_digit('3'))
        self.assertEqual("080 7031 30", formatter.input_digit('0'))
        self.assertEqual("080 7031 300", formatter.input_digit('0'))
        self.assertEqual("080 7031 3000", formatter.input_digit('0'))

    def testAYTFGBPremiumRate(self):
        formatter = AsYouTypeFormatter("GB")
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("09", formatter.input_digit('9'))
        self.assertEqual("090", formatter.input_digit('0'))
        self.assertEqual("090 7", formatter.input_digit('7'))
        self.assertEqual("090 70", formatter.input_digit('0'))
        self.assertEqual("090 703", formatter.input_digit('3'))
        self.assertEqual("090 7031", formatter.input_digit('1'))
        self.assertEqual("090 7031 3", formatter.input_digit('3'))
        self.assertEqual("090 7031 30", formatter.input_digit('0'))
        self.assertEqual("090 7031 300", formatter.input_digit('0'))
        self.assertEqual("090 7031 3000", formatter.input_digit('0'))

    def testAYTFNZMobile(self):
        formatter = AsYouTypeFormatter("NZ")
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("02", formatter.input_digit('2'))
        self.assertEqual("021", formatter.input_digit('1'))
        self.assertEqual("02-11", formatter.input_digit('1'))
        self.assertEqual("02-112", formatter.input_digit('2'))
        # Note the unittest is using fake metadata which might produce non-ideal results.
        self.assertEqual("02-112 3", formatter.input_digit('3'))
        self.assertEqual("02-112 34", formatter.input_digit('4'))
        self.assertEqual("02-112 345", formatter.input_digit('5'))
        self.assertEqual("02-112 3456", formatter.input_digit('6'))

    def testAYTFDE(self):
        formatter = AsYouTypeFormatter("DE")
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("03", formatter.input_digit('3'))
        self.assertEqual("030", formatter.input_digit('0'))
        self.assertEqual("030/1", formatter.input_digit('1'))
        self.assertEqual("030/12", formatter.input_digit('2'))
        self.assertEqual("030/123", formatter.input_digit('3'))
        self.assertEqual("030/1234", formatter.input_digit('4'))

        # 04134 1234
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("04", formatter.input_digit('4'))
        self.assertEqual("041", formatter.input_digit('1'))
        self.assertEqual("041 3", formatter.input_digit('3'))
        self.assertEqual("041 34", formatter.input_digit('4'))
        self.assertEqual("04134 1", formatter.input_digit('1'))
        self.assertEqual("04134 12", formatter.input_digit('2'))
        self.assertEqual("04134 123", formatter.input_digit('3'))
        self.assertEqual("04134 1234", formatter.input_digit('4'))

        # 08021 2345
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("08", formatter.input_digit('8'))
        self.assertEqual("080", formatter.input_digit('0'))
        self.assertEqual("080 2", formatter.input_digit('2'))
        self.assertEqual("080 21", formatter.input_digit('1'))
        self.assertEqual("08021 2", formatter.input_digit('2'))
        self.assertEqual("08021 23", formatter.input_digit('3'))
        self.assertEqual("08021 234", formatter.input_digit('4'))
        self.assertEqual("08021 2345", formatter.input_digit('5'))

        # 00 1 650 253 2250
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("00", formatter.input_digit('0'))
        self.assertEqual("00 1 ", formatter.input_digit('1'))
        self.assertEqual("00 1 6", formatter.input_digit('6'))
        self.assertEqual("00 1 65", formatter.input_digit('5'))
        self.assertEqual("00 1 650", formatter.input_digit('0'))
        self.assertEqual("00 1 650 2", formatter.input_digit('2'))
        self.assertEqual("00 1 650 25", formatter.input_digit('5'))
        self.assertEqual("00 1 650 253", formatter.input_digit('3'))
        self.assertEqual("00 1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("00 1 650 253 22", formatter.input_digit('2'))
        self.assertEqual("00 1 650 253 222", formatter.input_digit('2'))
        self.assertEqual("00 1 650 253 2222", formatter.input_digit('2'))

    def testAYTFAR(self):
        formatter = AsYouTypeFormatter("AR")
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011", formatter.input_digit('1'))
        self.assertEqual("011 7", formatter.input_digit('7'))
        self.assertEqual("011 70", formatter.input_digit('0'))
        self.assertEqual("011 703", formatter.input_digit('3'))
        self.assertEqual("011 7031", formatter.input_digit('1'))
        self.assertEqual("011 7031-3", formatter.input_digit('3'))
        self.assertEqual("011 7031-30", formatter.input_digit('0'))
        self.assertEqual("011 7031-300", formatter.input_digit('0'))
        self.assertEqual("011 7031-3000", formatter.input_digit('0'))

    def testAYTFARMobile(self):
        formatter = AsYouTypeFormatter("AR")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+54 ", formatter.input_digit('4'))
        self.assertEqual("+54 9", formatter.input_digit('9'))
        self.assertEqual("+54 91", formatter.input_digit('1'))
        self.assertEqual("+54 9 11", formatter.input_digit('1'))
        self.assertEqual("+54 9 11 2", formatter.input_digit('2'))
        self.assertEqual("+54 9 11 23", formatter.input_digit('3'))
        self.assertEqual("+54 9 11 231", formatter.input_digit('1'))
        self.assertEqual("+54 9 11 2312", formatter.input_digit('2'))
        self.assertEqual("+54 9 11 2312 1", formatter.input_digit('1'))
        self.assertEqual("+54 9 11 2312 12", formatter.input_digit('2'))
        self.assertEqual("+54 9 11 2312 123", formatter.input_digit('3'))
        self.assertEqual("+54 9 11 2312 1234", formatter.input_digit('4'))

    def testAYTFKR(self):
        # +82 51 234 5678
        formatter = AsYouTypeFormatter("KR")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+82 ", formatter.input_digit('2'))
        self.assertEqual("+82 5", formatter.input_digit('5'))
        self.assertEqual("+82 51", formatter.input_digit('1'))
        self.assertEqual("+82 51-2", formatter.input_digit('2'))
        self.assertEqual("+82 51-23", formatter.input_digit('3'))
        self.assertEqual("+82 51-234", formatter.input_digit('4'))
        self.assertEqual("+82 51-234-5", formatter.input_digit('5'))
        self.assertEqual("+82 51-234-56", formatter.input_digit('6'))
        self.assertEqual("+82 51-234-567", formatter.input_digit('7'))
        self.assertEqual("+82 51-234-5678", formatter.input_digit('8'))

        # +82 2 531 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+82 ", formatter.input_digit('2'))
        self.assertEqual("+82 2", formatter.input_digit('2'))
        self.assertEqual("+82 25", formatter.input_digit('5'))
        self.assertEqual("+82 2-53", formatter.input_digit('3'))
        self.assertEqual("+82 2-531", formatter.input_digit('1'))
        self.assertEqual("+82 2-531-5", formatter.input_digit('5'))
        self.assertEqual("+82 2-531-56", formatter.input_digit('6'))
        self.assertEqual("+82 2-531-567", formatter.input_digit('7'))
        self.assertEqual("+82 2-531-5678", formatter.input_digit('8'))

        # +82 2 3665 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+82 ", formatter.input_digit('2'))
        self.assertEqual("+82 2", formatter.input_digit('2'))
        self.assertEqual("+82 23", formatter.input_digit('3'))
        self.assertEqual("+82 2-36", formatter.input_digit('6'))
        self.assertEqual("+82 2-366", formatter.input_digit('6'))
        self.assertEqual("+82 2-3665", formatter.input_digit('5'))
        self.assertEqual("+82 2-3665-5", formatter.input_digit('5'))
        self.assertEqual("+82 2-3665-56", formatter.input_digit('6'))
        self.assertEqual("+82 2-3665-567", formatter.input_digit('7'))
        self.assertEqual("+82 2-3665-5678", formatter.input_digit('8'))

        # 02-114
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("02", formatter.input_digit('2'))
        self.assertEqual("021", formatter.input_digit('1'))
        self.assertEqual("02-11", formatter.input_digit('1'))
        self.assertEqual("02-114", formatter.input_digit('4'))

        # 02-1300
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("02", formatter.input_digit('2'))
        self.assertEqual("021", formatter.input_digit('1'))
        self.assertEqual("02-13", formatter.input_digit('3'))
        self.assertEqual("02-130", formatter.input_digit('0'))
        self.assertEqual("02-1300", formatter.input_digit('0'))

        # 011-456-7890
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011", formatter.input_digit('1'))
        self.assertEqual("011-4", formatter.input_digit('4'))
        self.assertEqual("011-45", formatter.input_digit('5'))
        self.assertEqual("011-456", formatter.input_digit('6'))
        self.assertEqual("011-456-7", formatter.input_digit('7'))
        self.assertEqual("011-456-78", formatter.input_digit('8'))
        self.assertEqual("011-456-789", formatter.input_digit('9'))
        self.assertEqual("011-456-7890", formatter.input_digit('0'))

        # 011-9876-7890
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011", formatter.input_digit('1'))
        self.assertEqual("011-9", formatter.input_digit('9'))
        self.assertEqual("011-98", formatter.input_digit('8'))
        self.assertEqual("011-987", formatter.input_digit('7'))
        self.assertEqual("011-9876", formatter.input_digit('6'))
        self.assertEqual("011-9876-7", formatter.input_digit('7'))
        self.assertEqual("011-9876-78", formatter.input_digit('8'))
        self.assertEqual("011-9876-789", formatter.input_digit('9'))
        self.assertEqual("011-9876-7890", formatter.input_digit('0'))

    def testAYTF_MX(self):
        formatter = AsYouTypeFormatter("MX")
        # +52 800 123 4567
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+52 ", formatter.input_digit('2'))
        self.assertEqual("+52 8", formatter.input_digit('8'))
        self.assertEqual("+52 80", formatter.input_digit('0'))
        self.assertEqual("+52 800", formatter.input_digit('0'))
        self.assertEqual("+52 800 1", formatter.input_digit('1'))
        self.assertEqual("+52 800 12", formatter.input_digit('2'))
        self.assertEqual("+52 800 123", formatter.input_digit('3'))
        self.assertEqual("+52 800 123 4", formatter.input_digit('4'))
        self.assertEqual("+52 800 123 45", formatter.input_digit('5'))
        self.assertEqual("+52 800 123 456", formatter.input_digit('6'))
        self.assertEqual("+52 800 123 4567", formatter.input_digit('7'))

        # +52 55 1234 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+52 ", formatter.input_digit('2'))
        self.assertEqual("+52 5", formatter.input_digit('5'))
        self.assertEqual("+52 55", formatter.input_digit('5'))
        self.assertEqual("+52 55 1", formatter.input_digit('1'))
        self.assertEqual("+52 55 12", formatter.input_digit('2'))
        self.assertEqual("+52 55 123", formatter.input_digit('3'))
        self.assertEqual("+52 55 1234", formatter.input_digit('4'))
        self.assertEqual("+52 55 1234 5", formatter.input_digit('5'))
        self.assertEqual("+52 55 1234 56", formatter.input_digit('6'))
        self.assertEqual("+52 55 1234 567", formatter.input_digit('7'))
        self.assertEqual("+52 55 1234 5678", formatter.input_digit('8'))

        # +52 212 345 6789
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+52 ", formatter.input_digit('2'))
        self.assertEqual("+52 2", formatter.input_digit('2'))
        self.assertEqual("+52 21", formatter.input_digit('1'))
        self.assertEqual("+52 212", formatter.input_digit('2'))
        self.assertEqual("+52 212 3", formatter.input_digit('3'))
        self.assertEqual("+52 212 34", formatter.input_digit('4'))
        self.assertEqual("+52 212 345", formatter.input_digit('5'))
        self.assertEqual("+52 212 345 6", formatter.input_digit('6'))
        self.assertEqual("+52 212 345 67", formatter.input_digit('7'))
        self.assertEqual("+52 212 345 678", formatter.input_digit('8'))
        self.assertEqual("+52 212 345 6789", formatter.input_digit('9'))

        # +52 1 55 1234 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+52 ", formatter.input_digit('2'))
        self.assertEqual("+52 1", formatter.input_digit('1'))
        self.assertEqual("+52 15", formatter.input_digit('5'))
        self.assertEqual("+52 1 55", formatter.input_digit('5'))
        self.assertEqual("+52 1 55 1", formatter.input_digit('1'))
        self.assertEqual("+52 1 55 12", formatter.input_digit('2'))
        self.assertEqual("+52 1 55 123", formatter.input_digit('3'))
        self.assertEqual("+52 1 55 1234", formatter.input_digit('4'))
        self.assertEqual("+52 1 55 1234 5", formatter.input_digit('5'))
        self.assertEqual("+52 1 55 1234 56", formatter.input_digit('6'))
        self.assertEqual("+52 1 55 1234 567", formatter.input_digit('7'))
        self.assertEqual("+52 1 55 1234 5678", formatter.input_digit('8'))

        # +52 1 541 234 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+52 ", formatter.input_digit('2'))
        self.assertEqual("+52 1", formatter.input_digit('1'))
        self.assertEqual("+52 15", formatter.input_digit('5'))
        self.assertEqual("+52 1 54", formatter.input_digit('4'))
        self.assertEqual("+52 1 541", formatter.input_digit('1'))
        self.assertEqual("+52 1 541 2", formatter.input_digit('2'))
        self.assertEqual("+52 1 541 23", formatter.input_digit('3'))
        self.assertEqual("+52 1 541 234", formatter.input_digit('4'))
        self.assertEqual("+52 1 541 234 5", formatter.input_digit('5'))
        self.assertEqual("+52 1 541 234 56", formatter.input_digit('6'))
        self.assertEqual("+52 1 541 234 567", formatter.input_digit('7'))
        self.assertEqual("+52 1 541 234 5678", formatter.input_digit('8'))

    def testAYTF_International_Toll_Free(self):
        formatter = AsYouTypeFormatter("US")
        # +800 1234 5678
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+80", formatter.input_digit('0'))
        self.assertEqual("+800 ", formatter.input_digit('0'))
        self.assertEqual("+800 1", formatter.input_digit('1'))
        self.assertEqual("+800 12", formatter.input_digit('2'))
        self.assertEqual("+800 123", formatter.input_digit('3'))
        self.assertEqual("+800 1234", formatter.input_digit('4'))
        self.assertEqual("+800 1234 5", formatter.input_digit('5'))
        self.assertEqual("+800 1234 56", formatter.input_digit('6'))
        self.assertEqual("+800 1234 567", formatter.input_digit('7'))
        self.assertEqual("+800 1234 5678", formatter.input_digit('8'))
        self.assertEqual("+800123456789", formatter.input_digit('9'))

    def testAYTFMultipleLeadingDigitPatterns(self):
        # +81 50 2345 6789
        formatter = AsYouTypeFormatter("JP")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+81 ", formatter.input_digit('1'))
        self.assertEqual("+81 5", formatter.input_digit('5'))
        self.assertEqual("+81 50", formatter.input_digit('0'))
        self.assertEqual("+81 50 2", formatter.input_digit('2'))
        self.assertEqual("+81 50 23", formatter.input_digit('3'))
        self.assertEqual("+81 50 234", formatter.input_digit('4'))
        self.assertEqual("+81 50 2345", formatter.input_digit('5'))
        self.assertEqual("+81 50 2345 6", formatter.input_digit('6'))
        self.assertEqual("+81 50 2345 67", formatter.input_digit('7'))
        self.assertEqual("+81 50 2345 678", formatter.input_digit('8'))
        self.assertEqual("+81 50 2345 6789", formatter.input_digit('9'))

        # +81 222 12 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+81 ", formatter.input_digit('1'))
        self.assertEqual("+81 2", formatter.input_digit('2'))
        self.assertEqual("+81 22", formatter.input_digit('2'))
        self.assertEqual("+81 22 2", formatter.input_digit('2'))
        self.assertEqual("+81 22 21", formatter.input_digit('1'))
        self.assertEqual("+81 2221 2", formatter.input_digit('2'))
        self.assertEqual("+81 222 12 5", formatter.input_digit('5'))
        self.assertEqual("+81 222 12 56", formatter.input_digit('6'))
        self.assertEqual("+81 222 12 567", formatter.input_digit('7'))
        self.assertEqual("+81 222 12 5678", formatter.input_digit('8'))

        # 011113
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("01", formatter.input_digit('1'))
        self.assertEqual("011", formatter.input_digit('1'))
        self.assertEqual("011 1", formatter.input_digit('1'))
        self.assertEqual("011 11", formatter.input_digit('1'))
        self.assertEqual("011113", formatter.input_digit('3'))

        # +81 3332 2 5678
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+81 ", formatter.input_digit('1'))
        self.assertEqual("+81 3", formatter.input_digit('3'))
        self.assertEqual("+81 33", formatter.input_digit('3'))
        self.assertEqual("+81 33 3", formatter.input_digit('3'))
        self.assertEqual("+81 3332", formatter.input_digit('2'))
        self.assertEqual("+81 3332 2", formatter.input_digit('2'))
        self.assertEqual("+81 3332 2 5", formatter.input_digit('5'))
        self.assertEqual("+81 3332 2 56", formatter.input_digit('6'))
        self.assertEqual("+81 3332 2 567", formatter.input_digit('7'))
        self.assertEqual("+81 3332 2 5678", formatter.input_digit('8'))

    def testAYTFLongIDD_AU(self):
        formatter = AsYouTypeFormatter("AU")
        # 0011 1 650 253 2250
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("00", formatter.input_digit('0'))
        self.assertEqual("001", formatter.input_digit('1'))
        self.assertEqual("0011", formatter.input_digit('1'))
        self.assertEqual("0011 1 ", formatter.input_digit('1'))
        self.assertEqual("0011 1 6", formatter.input_digit('6'))
        self.assertEqual("0011 1 65", formatter.input_digit('5'))
        self.assertEqual("0011 1 650", formatter.input_digit('0'))
        self.assertEqual("0011 1 650 2", formatter.input_digit('2'))
        self.assertEqual("0011 1 650 25", formatter.input_digit('5'))
        self.assertEqual("0011 1 650 253", formatter.input_digit('3'))
        self.assertEqual("0011 1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("0011 1 650 253 22", formatter.input_digit('2'))
        self.assertEqual("0011 1 650 253 222", formatter.input_digit('2'))
        self.assertEqual("0011 1 650 253 2222", formatter.input_digit('2'))

        # 0011 81 3332 2 5678
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("00", formatter.input_digit('0'))
        self.assertEqual("001", formatter.input_digit('1'))
        self.assertEqual("0011", formatter.input_digit('1'))
        self.assertEqual("00118", formatter.input_digit('8'))
        self.assertEqual("0011 81 ", formatter.input_digit('1'))
        self.assertEqual("0011 81 3", formatter.input_digit('3'))
        self.assertEqual("0011 81 33", formatter.input_digit('3'))
        self.assertEqual("0011 81 33 3", formatter.input_digit('3'))
        self.assertEqual("0011 81 3332", formatter.input_digit('2'))
        self.assertEqual("0011 81 3332 2", formatter.input_digit('2'))
        self.assertEqual("0011 81 3332 2 5", formatter.input_digit('5'))
        self.assertEqual("0011 81 3332 2 56", formatter.input_digit('6'))
        self.assertEqual("0011 81 3332 2 567", formatter.input_digit('7'))
        self.assertEqual("0011 81 3332 2 5678", formatter.input_digit('8'))

        # 0011 244 250 253 222
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("00", formatter.input_digit('0'))
        self.assertEqual("001", formatter.input_digit('1'))
        self.assertEqual("0011", formatter.input_digit('1'))
        self.assertEqual("00112", formatter.input_digit('2'))
        self.assertEqual("001124", formatter.input_digit('4'))
        self.assertEqual("0011 244 ", formatter.input_digit('4'))
        self.assertEqual("0011 244 2", formatter.input_digit('2'))
        self.assertEqual("0011 244 25", formatter.input_digit('5'))
        self.assertEqual("0011 244 250", formatter.input_digit('0'))
        self.assertEqual("0011 244 250 2", formatter.input_digit('2'))
        self.assertEqual("0011 244 250 25", formatter.input_digit('5'))
        self.assertEqual("0011 244 250 253", formatter.input_digit('3'))
        self.assertEqual("0011 244 250 253 2", formatter.input_digit('2'))
        self.assertEqual("0011 244 250 253 22", formatter.input_digit('2'))
        self.assertEqual("0011 244 250 253 222", formatter.input_digit('2'))

    def testAYTFLongIDD_KR(self):
        formatter = AsYouTypeFormatter("KR")
        # 00300 1 650 253 2222
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("00", formatter.input_digit('0'))
        self.assertEqual("003", formatter.input_digit('3'))
        self.assertEqual("0030", formatter.input_digit('0'))
        self.assertEqual("00300", formatter.input_digit('0'))
        self.assertEqual("00300 1 ", formatter.input_digit('1'))
        self.assertEqual("00300 1 6", formatter.input_digit('6'))
        self.assertEqual("00300 1 65", formatter.input_digit('5'))
        self.assertEqual("00300 1 650", formatter.input_digit('0'))
        self.assertEqual("00300 1 650 2", formatter.input_digit('2'))
        self.assertEqual("00300 1 650 25", formatter.input_digit('5'))
        self.assertEqual("00300 1 650 253", formatter.input_digit('3'))
        self.assertEqual("00300 1 650 253 2", formatter.input_digit('2'))
        self.assertEqual("00300 1 650 253 22", formatter.input_digit('2'))
        self.assertEqual("00300 1 650 253 222", formatter.input_digit('2'))
        self.assertEqual("00300 1 650 253 2222", formatter.input_digit('2'))

    def testAYTFLongNDD_KR(self):
        formatter = AsYouTypeFormatter("KR")
        # 08811-9876-7890
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("08", formatter.input_digit('8'))
        self.assertEqual("088", formatter.input_digit('8'))
        self.assertEqual("0881", formatter.input_digit('1'))
        self.assertEqual("08811", formatter.input_digit('1'))
        self.assertEqual("08811-9", formatter.input_digit('9'))
        self.assertEqual("08811-98", formatter.input_digit('8'))
        self.assertEqual("08811-987", formatter.input_digit('7'))
        self.assertEqual("08811-9876", formatter.input_digit('6'))
        self.assertEqual("08811-9876-7", formatter.input_digit('7'))
        self.assertEqual("08811-9876-78", formatter.input_digit('8'))
        self.assertEqual("08811-9876-789", formatter.input_digit('9'))
        self.assertEqual("08811-9876-7890", formatter.input_digit('0'))

        # 08500 11-9876-7890
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("08", formatter.input_digit('8'))
        self.assertEqual("085", formatter.input_digit('5'))
        self.assertEqual("0850", formatter.input_digit('0'))
        self.assertEqual("08500 ", formatter.input_digit('0'))
        self.assertEqual("08500 1", formatter.input_digit('1'))
        self.assertEqual("08500 11", formatter.input_digit('1'))
        self.assertEqual("08500 11-9", formatter.input_digit('9'))
        self.assertEqual("08500 11-98", formatter.input_digit('8'))
        self.assertEqual("08500 11-987", formatter.input_digit('7'))
        self.assertEqual("08500 11-9876", formatter.input_digit('6'))
        self.assertEqual("08500 11-9876-7", formatter.input_digit('7'))
        self.assertEqual("08500 11-9876-78", formatter.input_digit('8'))
        self.assertEqual("08500 11-9876-789", formatter.input_digit('9'))
        self.assertEqual("08500 11-9876-7890", formatter.input_digit('0'))

    def testAYTFLongNDD_SG(self):
        formatter = AsYouTypeFormatter("SG")
        # 777777 9876 7890
        self.assertEqual("7", formatter.input_digit('7'))
        self.assertEqual("77", formatter.input_digit('7'))
        self.assertEqual("777", formatter.input_digit('7'))
        self.assertEqual("7777", formatter.input_digit('7'))
        self.assertEqual("77777", formatter.input_digit('7'))
        self.assertEqual("777777 ", formatter.input_digit('7'))
        self.assertEqual("777777 9", formatter.input_digit('9'))
        self.assertEqual("777777 98", formatter.input_digit('8'))
        self.assertEqual("777777 987", formatter.input_digit('7'))
        self.assertEqual("777777 9876", formatter.input_digit('6'))
        self.assertEqual("777777 9876 7", formatter.input_digit('7'))
        self.assertEqual("777777 9876 78", formatter.input_digit('8'))
        self.assertEqual("777777 9876 789", formatter.input_digit('9'))
        self.assertEqual("777777 9876 7890", formatter.input_digit('0'))

    def testAYTFNoNationalPrefixFormattingRule(self):
        formatter = AsYouTypeFormatter("AO")
        self.assertEqual("3", formatter.input_digit('3'))
        self.assertEqual("33", formatter.input_digit('3'))
        self.assertEqual("333", formatter.input_digit('3'))
        self.assertEqual("333 3", formatter.input_digit('3'))
        self.assertEqual("333 33", formatter.input_digit('3'))
        self.assertEqual("333 333", formatter.input_digit('3'))

    def testAYTFShortNumberFormattingFix_AU(self):
        # For Australia, the national prefix is not optional when formatting.
        formatter = AsYouTypeFormatter("AU")

        # 1234567890 - For leading digit 1, the national prefix formatting
        # rule has first group only.
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("12", formatter.input_digit('2'))
        self.assertEqual("123", formatter.input_digit('3'))
        self.assertEqual("1234", formatter.input_digit('4'))
        self.assertEqual("1234 5", formatter.input_digit('5'))
        self.assertEqual("1234 56", formatter.input_digit('6'))
        self.assertEqual("1234 567", formatter.input_digit('7'))
        self.assertEqual("1234 567 8", formatter.input_digit('8'))
        self.assertEqual("1234 567 89", formatter.input_digit('9'))
        self.assertEqual("1234 567 890", formatter.input_digit('0'))

        # +61 1234 567 890 - Test the same number, but with the country code.
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+6", formatter.input_digit('6'))
        self.assertEqual("+61 ", formatter.input_digit('1'))
        self.assertEqual("+61 1", formatter.input_digit('1'))
        self.assertEqual("+61 12", formatter.input_digit('2'))
        self.assertEqual("+61 123", formatter.input_digit('3'))
        self.assertEqual("+61 1234", formatter.input_digit('4'))
        self.assertEqual("+61 1234 5", formatter.input_digit('5'))
        self.assertEqual("+61 1234 56", formatter.input_digit('6'))
        self.assertEqual("+61 1234 567", formatter.input_digit('7'))
        self.assertEqual("+61 1234 567 8", formatter.input_digit('8'))
        self.assertEqual("+61 1234 567 89", formatter.input_digit('9'))
        self.assertEqual("+61 1234 567 890", formatter.input_digit('0'))

        # 212345678 - For leading digit 2, the national prefix formatting rule
        # puts the national prefix before the first group.
        formatter.clear()
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("02", formatter.input_digit('2'))
        self.assertEqual("021", formatter.input_digit('1'))
        self.assertEqual("02 12", formatter.input_digit('2'))
        self.assertEqual("02 123", formatter.input_digit('3'))
        self.assertEqual("02 1234", formatter.input_digit('4'))
        self.assertEqual("02 1234 5", formatter.input_digit('5'))
        self.assertEqual("02 1234 56", formatter.input_digit('6'))
        self.assertEqual("02 1234 567", formatter.input_digit('7'))
        self.assertEqual("02 1234 5678", formatter.input_digit('8'))

        # 212345678 - Test the same number, but without the leading 0.
        formatter.clear()
        self.assertEqual("2", formatter.input_digit('2'))
        self.assertEqual("21", formatter.input_digit('1'))
        self.assertEqual("212", formatter.input_digit('2'))
        self.assertEqual("2123", formatter.input_digit('3'))
        self.assertEqual("21234", formatter.input_digit('4'))
        self.assertEqual("212345", formatter.input_digit('5'))
        self.assertEqual("2123456", formatter.input_digit('6'))
        self.assertEqual("21234567", formatter.input_digit('7'))
        self.assertEqual("212345678", formatter.input_digit('8'))

        # +61 2 1234 5678 - Test the same number, but with the country code.
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+6", formatter.input_digit('6'))
        self.assertEqual("+61 ", formatter.input_digit('1'))
        self.assertEqual("+61 2", formatter.input_digit('2'))
        self.assertEqual("+61 21", formatter.input_digit('1'))
        self.assertEqual("+61 2 12", formatter.input_digit('2'))
        self.assertEqual("+61 2 123", formatter.input_digit('3'))
        self.assertEqual("+61 2 1234", formatter.input_digit('4'))
        self.assertEqual("+61 2 1234 5", formatter.input_digit('5'))
        self.assertEqual("+61 2 1234 56", formatter.input_digit('6'))
        self.assertEqual("+61 2 1234 567", formatter.input_digit('7'))
        self.assertEqual("+61 2 1234 5678", formatter.input_digit('8'))

    def testAYTFShortNumberFormattingFix_KR(self):
        # For Korea, the national prefix is not optional when formatting, and
        # the national prefix formatting rule doesn't consist of only the
        # first group.
        formatter = AsYouTypeFormatter("KR")

        # 111
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("11", formatter.input_digit('1'))
        self.assertEqual("111", formatter.input_digit('1'))

        # 114
        formatter.clear()
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("11", formatter.input_digit('1'))
        self.assertEqual("114", formatter.input_digit('4'))

        # 13121234 - Test a mobile number without the national prefix. Even
        # though it is not an emergency number, it should be formatted as a
        # block.
        formatter.clear()
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("13", formatter.input_digit('3'))
        self.assertEqual("131", formatter.input_digit('1'))
        self.assertEqual("1312", formatter.input_digit('2'))
        self.assertEqual("13121", formatter.input_digit('1'))
        self.assertEqual("131212", formatter.input_digit('2'))
        self.assertEqual("1312123", formatter.input_digit('3'))
        self.assertEqual("13121234", formatter.input_digit('4'))

        # +82 131-2-1234 - Test the same number, but with the country code.
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+82 ", formatter.input_digit('2'))
        self.assertEqual("+82 1", formatter.input_digit('1'))
        self.assertEqual("+82 13", formatter.input_digit('3'))
        self.assertEqual("+82 131", formatter.input_digit('1'))
        self.assertEqual("+82 131-2", formatter.input_digit('2'))
        self.assertEqual("+82 131-2-1", formatter.input_digit('1'))
        self.assertEqual("+82 131-2-12", formatter.input_digit('2'))
        self.assertEqual("+82 131-2-123", formatter.input_digit('3'))
        self.assertEqual("+82 131-2-1234", formatter.input_digit('4'))

    def testAYTFShortNumberFormattingFix_MX(self):
        # For Mexico, the national prefix is optional when formatting.
        formatter = AsYouTypeFormatter("MX")

        # 911
        self.assertEqual("9", formatter.input_digit('9'))
        self.assertEqual("91", formatter.input_digit('1'))
        self.assertEqual("911", formatter.input_digit('1'))

        # 800 123 4567 - Test a toll-free number, which should have a
        # formatting rule applied to it even though it doesn't begin with the
        # national prefix.
        formatter.clear()
        self.assertEqual("8", formatter.input_digit('8'))
        self.assertEqual("80", formatter.input_digit('0'))
        self.assertEqual("800", formatter.input_digit('0'))
        self.assertEqual("800 1", formatter.input_digit('1'))
        self.assertEqual("800 12", formatter.input_digit('2'))
        self.assertEqual("800 123", formatter.input_digit('3'))
        self.assertEqual("800 123 4", formatter.input_digit('4'))
        self.assertEqual("800 123 45", formatter.input_digit('5'))
        self.assertEqual("800 123 456", formatter.input_digit('6'))
        self.assertEqual("800 123 4567", formatter.input_digit('7'))

        # +52 800 123 4567 - Test the same number, but with the country code.
        formatter.clear()
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+5", formatter.input_digit('5'))
        self.assertEqual("+52 ", formatter.input_digit('2'))
        self.assertEqual("+52 8", formatter.input_digit('8'))
        self.assertEqual("+52 80", formatter.input_digit('0'))
        self.assertEqual("+52 800", formatter.input_digit('0'))
        self.assertEqual("+52 800 1", formatter.input_digit('1'))
        self.assertEqual("+52 800 12", formatter.input_digit('2'))
        self.assertEqual("+52 800 123", formatter.input_digit('3'))
        self.assertEqual("+52 800 123 4", formatter.input_digit('4'))
        self.assertEqual("+52 800 123 45", formatter.input_digit('5'))
        self.assertEqual("+52 800 123 456", formatter.input_digit('6'))
        self.assertEqual("+52 800 123 4567", formatter.input_digit('7'))

    def testAYTFNoNationalPrefix(self):
        formatter = AsYouTypeFormatter("IT")
        self.assertEqual("3", formatter.input_digit('3'))
        self.assertEqual("33", formatter.input_digit('3'))
        self.assertEqual("333", formatter.input_digit('3'))
        self.assertEqual("333 3", formatter.input_digit('3'))
        self.assertEqual("333 33", formatter.input_digit('3'))
        self.assertEqual("333 333", formatter.input_digit('3'))

    def testAYTFShortNumberFormattingFix_US(self):
        # For the US, an initial 1 is treated specially.
        formatter = AsYouTypeFormatter("US")

        # 101 - Test that the initial 1 is not treated as a national prefix.
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("10", formatter.input_digit('0'))
        self.assertEqual("101", formatter.input_digit('1'))

        # 112 - Test that the initial 1 is not treated as a national prefix.
        formatter.clear()
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("11", formatter.input_digit('1'))
        self.assertEqual("112", formatter.input_digit('2'))

        # 122 - Test that the initial 1 is treated as a national prefix.
        formatter.clear()
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("12", formatter.input_digit('2'))
        self.assertEqual("1 22", formatter.input_digit('2'))

    def testAYTFClearNDDAfterIDDExtraction(self):
        formatter = AsYouTypeFormatter("KR")

        # Check that when we have successfully extracted an IDD, the
        # previously extracted NDD is cleared since it is no longer valid.
        self.assertEqual("0", formatter.input_digit('0'))
        self.assertEqual("00", formatter.input_digit('0'))
        self.assertEqual("007", formatter.input_digit('7'))
        self.assertEqual("0070", formatter.input_digit('0'))
        self.assertEqual("00700", formatter.input_digit('0'))
        self.assertEqual("0", formatter._extracted_national_prefix)

        # Once the IDD "00700" has been extracted, it no longer makes sense
        # for the initial "0" to be treated as an NDD.
        self.assertEqual("00700 1 ", formatter.input_digit('1'))
        self.assertEqual("", formatter._extracted_national_prefix)

        self.assertEqual("00700 1 2", formatter.input_digit('2'))
        self.assertEqual("00700 1 23", formatter.input_digit('3'))
        self.assertEqual("00700 1 234", formatter.input_digit('4'))
        self.assertEqual("00700 1 234 5", formatter.input_digit('5'))
        self.assertEqual("00700 1 234 56", formatter.input_digit('6'))
        self.assertEqual("00700 1 234 567", formatter.input_digit('7'))
        self.assertEqual("00700 1 234 567 8", formatter.input_digit('8'))
        self.assertEqual("00700 1 234 567 89", formatter.input_digit('9'))
        self.assertEqual("00700 1 234 567 890", formatter.input_digit('0'))
        self.assertEqual("00700 1 234 567 8901", formatter.input_digit('1'))
        self.assertEqual("00700123456789012", formatter.input_digit('2'))
        self.assertEqual("007001234567890123", formatter.input_digit('3'))
        self.assertEqual("0070012345678901234", formatter.input_digit('4'))
        self.assertEqual("00700123456789012345", formatter.input_digit('5'))
        self.assertEqual("007001234567890123456", formatter.input_digit('6'))
        self.assertEqual("0070012345678901234567", formatter.input_digit('7'))

    def testAYTFNumberPatternsBecomingInvalidShouldNotResultInDigitLoss(self):
        formatter = AsYouTypeFormatter("CN")
        self.assertEqual("+", formatter.input_digit('+'))
        self.assertEqual("+8", formatter.input_digit('8'))
        self.assertEqual("+86 ", formatter.input_digit('6'))
        self.assertEqual("+86 9", formatter.input_digit('9'))
        self.assertEqual("+86 98", formatter.input_digit('8'))
        self.assertEqual("+86 988", formatter.input_digit('8'))
        self.assertEqual("+86 988 1", formatter.input_digit('1'))
        # Now the number pattern is no longer valid because there are multiple
        # leading digit patterns; when we try again to extract a country code
        # we should ensure we use the last leading digit pattern, rather than
        # the first one such that it *thinks* it's found a valid formatting
        # rule again.
        # https://github.com/googlei18n/libphonenumber/issues/437
        self.assertEqual("+8698812", formatter.input_digit('2'))
        self.assertEqual("+86988123", formatter.input_digit('3'))
        self.assertEqual("+869881234", formatter.input_digit('4'))
        self.assertEqual("+8698812345", formatter.input_digit('5'))

    def testAYTFShortNumberFormatting_AR(self):
        # Python version extra test: use real metadata
        formatter = AsYouTypeFormatter("AR")
        self.assertEqual("1", formatter.input_digit('1'))
        self.assertEqual("10", formatter.input_digit('0'))
        self.assertEqual("101", formatter.input_digit('1'))

    def testEdgeCases(self):
        # Python version extra tests for coverage
        metadataXX = PhoneMetadata(id='XX', country_code=384, international_prefix='011',
                                   general_desc=PhoneNumberDesc(national_number_pattern='\\d{10}'),
                                   fixed_line=PhoneNumberDesc(national_number_pattern='NA'),
                                   mobile=PhoneNumberDesc(national_number_pattern='NA'),
                                   national_prefix=u("0"),
                                   national_prefix_for_parsing=u("0"),
                                   number_format=[NumberFormat(pattern='([135][246]|[246][123])(\\d{4})(\\d{4})',
                                                               format=u("\\1 \\2 \\3"),
                                                               leading_digits_pattern=['[1-59]|[78]0'],
                                                               national_prefix_formatting_rule=u("\\1"))])
        PhoneMetadata._region_metadata['XX'] = metadataXX
        phonenumberutil.SUPPORTED_REGIONS.add("XX")
        phonenumberutil.COUNTRY_CODE_TO_REGION_CODE[384] = ("XX",)
        formatter = AsYouTypeFormatter('XX')
        # A pattern with "|" in it doesn't get formatting
        self.assertEqual('1', formatter.input_digit('1'))
        self.assertEqual('12', formatter.input_digit('2'))
        self.assertEqual('123', formatter.input_digit('3'))
        self.assertEqual('1234', formatter.input_digit('4'))
        # Hit internal error arms
        self.assertEqual("1234", formatter._input_accrued_national_number())
        formatter._national_number = ""
        self.assertEqual("", formatter._input_accrued_national_number())
        # Restore normality
        del phonenumberutil.COUNTRY_CODE_TO_REGION_CODE[384]
        phonenumberutil.SUPPORTED_REGIONS.remove('XX')
        del PhoneMetadata._region_metadata['XX']



#!/usr/bin/env python
"""Script to read the libphonenumber per-prefix metadata and generate Python code.

Invocation:
  buildprefixdata.py [options] indir outfile module_prefix

Processes all of the per-prefix data under the given input directory and emit
generated Python code.

Options:
  --var XXX  : use this prefix for variable names in generated code
  --flat     : don't do per-locale processing
  --sep C    : expect metadata to be a list with C as separator
  --chunks N : divide metadata into N chunks
"""

# Based on original metadata data files from libphonenumber:
#     resources/geocoding/*/*.txt, resources/carrier/*/*.txt
# Copyright (C) 2011 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import sys
import glob
import re
import getopt
import datetime
import math

# Use the local code in preference to any pre-installed version
sys.path.insert(0, '../../python')

from phonenumbers.util import prnt, rpr

PREFIXDATA_CHUNK_SIZE = 10000
PREFIXDATA_SUFFIX = ".txt"
BLANK_LINE_RE = re.compile(r'^\s*$', re.UNICODE)
COMMENT_LINE_RE = re.compile(r'^\s*#.*$', re.UNICODE)
DATA_LINE_RE = re.compile(r'^\+?(?P<prefix>\d+)\|(?P<stringdata>.*)$', re.UNICODE)

# Boilerplate header
PREFIXDATA_LOCALE_FILE_PROLOG = '''"""Per-prefix data, mapping each prefix to a dict of locale:name.

Auto-generated file, do not edit by hand.
"""
from %(module)s.util import u
'''
PREFIXDATA_FILE_PROLOG = '''"""Per-prefix data, mapping each prefix to a name.

Auto-generated file, do not edit by hand.
"""
from %(module)s.util import u
'''

# Copyright notice covering the XML metadata; include current year.
COPYRIGHT_NOTICE = """# Copyright (C) 2011-%s The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" % datetime.datetime.now().year


def load_locale_prefixdata_file(prefixdata, filename, locale=None, overall_prefix=None, separator=None):
    """Load per-prefix data from the given file, for the given locale and prefix.

    We assume that this file:
     - is encoded in UTF-8
     - may have comment lines (starting with #) and blank lines
     - has data lines of the form '<prefix>|<stringdata>'
     - contains only data for prefixes that are extensions of the filename.

    If overall_prefix is specified, lines are checked to ensure their prefix falls within this value.

    If locale is specified, prefixdata[prefix][locale] is filled in; otherwise, just prefixdata[prefix].

    If separator is specified, the string data will be split on this separator, and the output values
    in the dict will be tuples of strings rather than strings.
    """
    with open(filename, "rb") as infile:
        lineno = 0
        for line in infile:
            uline = line.decode('utf-8')
            lineno += 1
            dm = DATA_LINE_RE.match(uline)
            if dm:
                prefix = dm.group('prefix')
                stringdata = dm.group('stringdata')
                if stringdata != stringdata.rstrip():
                    print ("%s:%d: Warning: stripping trailing whitespace" % (filename, lineno))
                    stringdata = stringdata.rstrip()
                if overall_prefix is not None and not prefix.startswith(overall_prefix):
                    raise Exception("%s:%d: Prefix %s is not within %s" %
                                    (filename, lineno, prefix, overall_prefix))
                if separator is not None:
                    stringdata = tuple(stringdata.split(separator))
                if prefix not in prefixdata:
                    prefixdata[prefix] = {}
                if locale is not None:
                    prefixdata[prefix][locale] = stringdata
                else:
                    prefixdata[prefix] = stringdata
            elif BLANK_LINE_RE.match(uline):
                pass
            elif COMMENT_LINE_RE.match(uline):
                pass
            else:
                raise Exception("%s:%d: Unexpected line format: %s" %
                                (filename, lineno, line))


def load_locale_prefixdata(indir, separator=None):
    """Load per-prefix data from the given top-level directory.

    Prefix data is assumed to be held in files <indir>/<locale>/<prefix>.txt.
    The same prefix may occur in multiple files, giving the prefix's description
    in different locales.
    """
    prefixdata = {}  # prefix => dict mapping locale to description
    for locale in os.listdir(indir):
        if not os.path.isdir(os.path.join(indir, locale)):
            continue
        for filename in glob.glob(os.path.join(indir, locale, "*%s" % PREFIXDATA_SUFFIX)):
            overall_prefix, ext = os.path.splitext(os.path.basename(filename))
            load_locale_prefixdata_file(prefixdata, filename, locale, overall_prefix, separator)
    return prefixdata


def _stable_dict_repr(strdict):
    """Return a repr() for a dict keyed by a string, in sorted key order"""
    lines = []
    for key in sorted(strdict.keys()):
        lines.append("'%s': %s" % (key, rpr(strdict[key])))
    return "{%s}" % ", ".join(lines)


def _tuple_repr(data):
    """Return a repr() for a list/tuple"""
    if len(data) == 1:
        return "(%s,)" % rpr(data[0])
    else:
        return "(%s)" % ", ".join([rpr(x) for x in data])


def output_prefixdata_code(prefixdata, outfilename, module_prefix, varprefix, per_locale, chunks):
    """Output the per-prefix data in Python form to the given file """
    sorted_keys = sorted(prefixdata.keys())
    total_keys = len(sorted_keys)
    if chunks == -1:
        chunk_size = PREFIXDATA_CHUNK_SIZE
        total_chunks = int(math.ceil(total_keys / float(chunk_size)))
    else:
        chunk_size = int(math.ceil(total_keys / float(chunks)))
        total_chunks = chunks

    outdirname = os.path.dirname(outfilename)
    longest_prefix = 0
    for chunk_num in range(total_chunks):
        chunk_index = chunk_size * chunk_num
        chunk_keys = sorted_keys[chunk_index:chunk_index + chunk_size]
        chunk_data = {}
        for key in chunk_keys:
            chunk_data[key] = prefixdata[key]
        chunk_file = os.path.join(outdirname, 'data%d.py' % chunk_num)
        chunk_longest = output_prefixdata_chunk(
            chunk_data, chunk_file, module_prefix, per_locale)
        if chunk_longest > longest_prefix:
            longest_prefix = chunk_longest

    with open(outfilename, "w") as outfile:
        if per_locale:
            prnt(PREFIXDATA_LOCALE_FILE_PROLOG % {'module': module_prefix}, file=outfile)
        else:
            prnt(PREFIXDATA_FILE_PROLOG % {'module': module_prefix}, file=outfile)
        prnt(COPYRIGHT_NOTICE, file=outfile)
        prnt("%s_DATA = {}" % varprefix, file=outfile)
        for chunk_num in range(total_chunks):
            prnt("from .data%d import data" % chunk_num, file=outfile)
            prnt("%s_DATA.update(data)" % varprefix, file=outfile)
        prnt("del data", file=outfile)
        prnt("%s_LONGEST_PREFIX = %d" % (varprefix, longest_prefix), file=outfile)


def output_prefixdata_chunk(prefixdata, outfilename, module_prefix, per_locale):
    with open(outfilename, "w") as outfile:
        longest_prefix = 0
        if per_locale:
            prnt(PREFIXDATA_LOCALE_FILE_PROLOG % {'module': module_prefix}, file=outfile)
        else:
            prnt(PREFIXDATA_FILE_PROLOG % {'module': module_prefix}, file=outfile)
        prnt(COPYRIGHT_NOTICE, file=outfile)
        prnt("data = {", file=outfile)
        for prefix in prefixdata.keys():
            if len(prefix) > longest_prefix:
                longest_prefix = len(prefix)
            if per_locale:
                prnt(" '%s':%s," % (prefix, _stable_dict_repr(prefixdata[prefix])), file=outfile)
            else:
                prnt(" '%s':%s," % (prefix, _tuple_repr(prefixdata[prefix])), file=outfile)
        prnt("}", file=outfile)
    return longest_prefix


def _standalone(argv):
    """Parse the given input directory and emit generated code."""
    varprefix = "GEOCODE"
    per_locale = True
    separator = None
    chunks = -1
    try:
        opts, args = getopt.getopt(argv, "hv:fs:c:", ("help", "var=", "flat", "sep=", "chunks="))
    except getopt.GetoptError:
        prnt(__doc__, file=sys.stderr)
        sys.exit(1)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            prnt(__doc__, file=sys.stderr)
            sys.exit(1)
        elif opt in ("-v", "--var"):
            varprefix = arg
        elif opt in ("-f", "--flat"):
            per_locale = False
        elif opt in ("-s", "--sep"):
            separator = arg
        elif opt in ("-c", "--chunks"):
            chunks = int(arg)
        else:
            prnt("Unknown option %s" % opt, file=sys.stderr)
            prnt(__doc__, file=sys.stderr)
            sys.exit(1)
    if len(args) != 3:
        prnt(__doc__, file=sys.stderr)
        sys.exit(1)
    if per_locale:
        prefixdata = load_locale_prefixdata(args[0], separator=separator)
    else:
        prefixdata = {}
        load_locale_prefixdata_file(prefixdata, args[0], separator=separator)
    output_prefixdata_code(prefixdata, args[1], args[2], varprefix, per_locale, chunks)


if __name__ == "__main__":
    _standalone(sys.argv[1:])



#!/usr/bin/env python
import doctest
import unittest
import phonenumbers
from phonenumbers import util
from phonenumbers import re_util
from phonenumbers import unicode_util
from phonenumbers import geocoder
from phonenumbers import carrier
from phonenumbers import timezone
from tests import *


class DocTest(unittest.TestCase):
    def testDocStrings(self):
        self.assertEqual(0, doctest.testmod(phonenumbers)[0])
        self.assertEqual(0, doctest.testmod(util)[0])
        self.assertEqual(0, doctest.testmod(re_util)[0])
        self.assertEqual(0, doctest.testmod(unicode_util)[0])
        self.assertEqual(0, doctest.testmod(geocoder)[0])
        self.assertEqual(0, doctest.testmod(carrier)[0])
        self.assertEqual(0, doctest.testmod(timezone)[0])


if __name__ == '__main__':
    unittest.main()
